// book.cpp

// includes
#include <cstdlib>
#include <cstring>
#include "uci.h"
#include "move.h"
#include "move_gen.h"
#include "option.h"
#include "book.h"

#ifdef WIN32
#pragma warning(disable : 4244)
#endif
// Level 2 conversion from 'int' to 'short', possible loss of data ->

// types
struct entry_t
    {
    uint64 key;
    uint16 move;
    uint16 count;
    uint16 n;
    uint16 sum;
    };

// variables
static FILE *book_file;
static int book_size;

// prototypes
static int find_pos(uint64 key);
static void read_entry(entry_t *entry, int n);
static uint64 read_integer(FILE *file, int size);

// functions

// book_init()
void book_init()
    {
    book_file = NULL;
    book_size = 0;
    }

void book_parameter()
    {
    // UCI options
    book_close();

    if(option_get_bool("OwnBook"))
        {
        book_open(option_get_string("Book Path"));
        }
    }

// book_open()
void book_open(const char file_name [])
    {
    book_file = fopen(file_name, "rb");

    if(book_file != NULL)
        {
        fseek(book_file, 0, SEEK_END);
        book_size = ftell(book_file) / 16;
        }
    }

// book_close()
void book_close()
    {
    if(book_file != NULL)
        fclose(book_file);
    book_file = NULL;
    }

// book_move()
int book_move(board_t *board)
    {
    int best_move;
    int best_score;
    int pos;
    entry_t entry[1];
    int move;
    int score;
    list_t list[1];
    int i;

    if(book_file != NULL && book_size != 0)
        {
        // draw a move according to a fixed probability distribution
        best_move = 0;
        best_score = 0;

        for (pos = find_pos(board->key); pos < book_size; pos++)
            {
            read_entry(entry, pos);

            if(entry->key != board->key)
                break;

            move = entry->move;
            score = entry->count;
            // pick this move?
            best_score += score;

            if(random(best_score) < score)
                best_move = move;
            }

        if(best_move != 0)
            {
            // convert PolyGlot move into Fruit move; TODO: handle promotes
            gen_legal_moves(list, board);

            for (i = 0; i < list->size; i++)
                {
                move = list->move[i];

                if((move & 07777) == best_move)
                    return move;
                }
            }
        }

    return 0;
    }

// find_pos()
static int find_pos(uint64 key)
    {
    int left, right, mid;
    entry_t entry[1];
    // binary search (finds the leftmost entry)
    left = 0;
    right = book_size - 1;

    while(left < right)
        {
        mid = (left + right) / 2;
        read_entry(entry, mid);

        if(key <= entry->key)
            {
            right = mid;
            }
        else
            {
            left = mid + 1;
            }
        }
    read_entry(entry, left);
    return (entry->key == key) ? left : book_size;
    }

// read_entry()
static void read_entry(entry_t *entry, int n)
    {
    fseek(book_file, n * 16, SEEK_SET);
    entry->key = read_integer(book_file, 8);
    entry->move = read_integer(book_file, 2);
    entry->count = read_integer(book_file, 2);
    entry->n = read_integer(book_file, 2);
    entry->sum = read_integer(book_file, 2);
    }

// read_integer()
static uint64 read_integer(FILE *file, int size)
    {
    uint64 n;
    int i;
    int b;
    n = 0;

    for (i = 0; i < size; i++)
        {
        b = fgetc(file);
        n = (n << 8) | b;
        }
    return n;
    }
// end of book.cpp
