// evaluate.cpp

// includes
#include <cstdlib>
#include "attack.h"
#include "init.h"
#include "material.h"
#include "move.h"
#include "option.h"
#include "pawn.h"
#include "piece.h"
#include "see.h"
#include "search.h"
#include "value.h"

// macros
#define THROUGH(piece) ((piece)==0)

// constants and variables
const int knight_outpost[2][64] =
	{
	0,	 0,  0,  0,  0,  0,  0,  0,
	0,	 0,  0,  0,  0,  0,  0,  0,
	2,	 2,  5,  5,  5,  5,  2,  2,
	3,	 3,  6, 10, 10,  6,  3,  3,
	3,	 4,  7, 10, 10,  7,  4,  3,
	3,	 5,  7,  7,  7,  7,  5,  3,
	2,	 3,  3,  3,  3,  3,  3,  2,
	0,	 0,  0,  0,  0,  0,  0,  0,

	0,	 0,  0,  0,  0,  0,  0,  0,
	2,	 3,  3,  3,  3,  3,  3,  2,
	3,	 5,  7,  7,  7,  7,  5,  3,
	3,	 4,  7, 10, 10,  7,  4,  3,
	3,	 3,  6, 10, 10,  6,  3,  3,
	2,	 2,  5,  5,  5,  5,  2,  2,
	0,	 0,  0,  0,  0,  0,  0,  0,
	0,	 0,  0,  0,  0,  0,  0,  0
	};

// "constants"
static const int king_attack_weight[16] =
    {
    0, 0, 128, 192, 224, 240, 248, 252, 254, 255, 256, 256, 256, 256, 256, 256,
    };

static const int s_space_weight[16] =
    {
    0, 2, 5, 8, 12, 15, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20,
    };

// variables
static int mobility_unit[2][256];
static int king_attack_unit[256];
bool king_is_safe[2];

// prototypes
static void eval_draw(const board_t *board, const material_info_t *mat_info, const pawn_info_t *pawn_info,
    int mul[2]);
static void eval_piece(const board_t *board, const material_info_t *mat_info, const pawn_info_t *pawn_info,
    int *opening, int *endgame);
static void eval_king(const board_t *board, const material_info_t *mat_info, int *opening, int *endgame);
static void eval_passer(const board_t *board, const pawn_info_t *pawn_info, int *opening, int *endgame);
static void eval_pattern(const board_t *board, int *opening, int *endgame);
static bool unstoppable_passer(const board_t *board, int pawn, int color);
static bool king_passer(const board_t *board, int pawn, int color);
static bool free_passer(const board_t *board, int pawn, int color);
static int pawn_att_dist(int pawn, int king, int color);
static int pawn_def_dist(int pawn, int king, int color);
static void draw_init_list(int list [], const board_t *board, int pawn_color);
static bool draw_kpkq(const int list [], int turn);
static bool draw_kpkr(const int list [], int turn);
static bool draw_kpkb(const int list [], int turn);
static bool draw_kpkn(const int list [], int turn);
static bool draw_knpk(const int list [], int turn);
static bool draw_krpkr(const int list [], int turn);
static bool draw_kbpkb(const int list [], int turn);
static int shelter_square(const board_t *board, const material_info_t *mat_info, int square, int color);
static int shelter_file(const board_t *board, int file, int rank, int color);
static int storm_file(const board_t *board, int file, int color);
static bool bishop_can_attack(const board_t *board, int to, int color);

// functions

// eval_init()
void eval_init()
    {
    int color;
    int piece;

    // mobility table
    for (color = 0; color < 2; color++)
        {
        for (piece = 0; piece < 256; piece++)
            {
            mobility_unit[color][piece] = 0;
            }
        }
    mobility_unit[0][0] = mobility_move;
    mobility_unit[0][((1 << 3) | (1 << 1))] = mobility_attack;
    mobility_unit[0][((1 << 4) | (1 << 1))] = mobility_attack;
    mobility_unit[0][((1 << 5) | (1 << 1))] = mobility_attack;
    mobility_unit[0][((1 << 6) | (1 << 1))] = mobility_attack;
    mobility_unit[0][(((1 << 5) | (1 << 6)) | (1 << 1))] = mobility_attack;
    mobility_unit[0][((1 << 7) | (1 << 1))] = mobility_attack;
    mobility_unit[0][((1 << 2) | (1 << 0))] = mobility_defense;
    mobility_unit[0][((1 << 4) | (1 << 0))] = mobility_defense;
    mobility_unit[0][((1 << 5) | (1 << 0))] = mobility_defense;
    mobility_unit[0][((1 << 6) | (1 << 0))] = mobility_defense;
    mobility_unit[0][(((1 << 5) | (1 << 6)) | (1 << 0))] = mobility_defense;
    mobility_unit[0][((1 << 7) | (1 << 0))] = mobility_defense;
    mobility_unit[1][0] = 1;
    mobility_unit[1][((1 << 2) | (1 << 0))] = mobility_attack;
    mobility_unit[1][((1 << 4) | (1 << 0))] = mobility_attack;
    mobility_unit[1][((1 << 5) | (1 << 0))] = mobility_attack;
    mobility_unit[1][((1 << 6) | (1 << 0))] = mobility_attack;
    mobility_unit[1][(((1 << 5) | (1 << 6)) | (1 << 0))] = mobility_attack;
    mobility_unit[1][((1 << 7) | (1 << 0))] = mobility_attack;
    mobility_unit[1][((1 << 3) | (1 << 1))] = mobility_defense;
    mobility_unit[1][((1 << 4) | (1 << 1))] = mobility_defense;
    mobility_unit[1][((1 << 5) | (1 << 1))] = mobility_defense;
    mobility_unit[1][((1 << 6) | (1 << 1))] = mobility_defense;
    mobility_unit[1][(((1 << 5) | (1 << 6)) | (1 << 1))] = mobility_defense;
    mobility_unit[1][((1 << 7) | (1 << 1))] = mobility_defense;

    // king_attack_unit[]
    for (piece = 0; piece < 256; piece++)
        {
        king_attack_unit[piece] = 0;
        }
    king_attack_unit[((1 << 4) | (1 << 0))] = 1;
    king_attack_unit[((1 << 5) | (1 << 0))] = 1;
    king_attack_unit[((1 << 6) | (1 << 0))] = 2;
    king_attack_unit[(((1 << 5) | (1 << 6)) | (1 << 0))] = 4;
    king_attack_unit[((1 << 4) | (1 << 1))] = 1;
    king_attack_unit[((1 << 5) | (1 << 1))] = 1;
    king_attack_unit[((1 << 6) | (1 << 1))] = 2;
    king_attack_unit[(((1 << 5) | (1 << 6)) | (1 << 1))] = 4;
    }

// eval()
int eval(board_t *board, int alpha, int beta, int thread_id)
    {
    int opening, endgame;
    material_info_t mat_info[1];
    pawn_info_t pawn_info[1];
    int mul[2];
    int phase;
    int eval;
    int wb, bb;
    int lazy_eval_score;
    // init
    opening = 0;
    endgame = 0;
    // material
    material_get_info(mat_info, board, thread_id);
    opening += mat_info->opening;
    endgame += mat_info->endgame;
    mul[0] = mat_info->mul[0];
    mul[1] = mat_info->mul[1];
    // PST
    opening += board->opening;
    endgame += board->endgame;
    // pawns
    pawn_get_info(pawn_info, board, thread_id);
    opening += pawn_info->opening;
    endgame += pawn_info->endgame;
    // draw
    eval_draw(board, mat_info, pawn_info, mul);

    if(mat_info->mul[0] < mul[0])
        mul[0] = mat_info->mul[0];

    if(mat_info->mul[1] < mul[1])
        mul[1] = mat_info->mul[1];

    if(mul[0] == 0 && mul[1] == 0)
        return 0;

    // eval
    // Tempo
    if(COLOR_IS_WHITE(board->turn))
        {
        opening += 20;
        endgame += 10;
        }
    else
        {
        opening -= 20;
        endgame -= 10;
        }
    eval_pattern(board, &opening, &endgame);
    eval_passer(board, pawn_info, &opening, &endgame);

    if(use_lazy_eval && board->piece_size[0] > 3 && board->piece_size[1] > 3)
        {
        phase = mat_info->phase;
        lazy_eval_score = ((opening * (256 - mat_info->phase)) + (endgame * mat_info->phase)) / 256;

        if(COLOR_IS_BLACK(board->turn))
            lazy_eval_score = -lazy_eval_score;

        if(lazy_eval_score - lazy_eval_margin >= beta)
            return (lazy_eval_score);

        if(lazy_eval_score + lazy_eval_margin <= alpha)
            return (lazy_eval_score);
        }
    // eval
    eval_piece(board, mat_info, pawn_info, &opening, &endgame);
    eval_king(board, mat_info, &opening, &endgame);
    // phase mix
    phase = mat_info->phase;
    eval = ((opening * (256 - phase)) + (endgame * phase)) / 256;

    // drawish bishop endgames
    if((mat_info->flags &(1 << 1)) != 0)
        {
        wb = board->piece[0][1];
        bb = board->piece[1][1];

        if(SQUARE_COLOR(wb) != SQUARE_COLOR(bb))
            {
            if(mul[0] == 16)
                mul[0] = 8; // 1/2

            if(mul[1] == 16)
                mul[1] = 8; // 1/2
            }
        }

    // draw bound
    if(eval > 0)
        {
        eval = (eval * mul[0]) / 16;
        }
    else if(eval < 0)
        {
        eval = (eval * mul[1]) / 16;
        }

    // value range
    if(eval < -(29744))
        eval = -(29744);

    if(eval > +(29744))
        eval = +(29744);

    // turn
    if(COLOR_IS_BLACK(board->turn))
        eval = -eval;
    return eval;
    }

// eval_draw()
static void eval_draw(const board_t *board, const material_info_t *mat_info, const pawn_info_t *pawn_info, int mul[2])
    {
    int color;
    int me, opp;
    int pawn, king;
    int pawn_file;
    int prom;
    int list[7 + 1];

    // draw patterns
    for (color = 0; color < 2; color++)
        {
        me = color;
        opp = COLOR_OPP(me);

        // KB*P+K* draw
        if((mat_info->cflags[me]&(1 << 0)) != 0)
            {
            pawn = pawn_info->single_file[me];

            if(pawn != 0)
                { // all pawns on one file
                pawn_file = SQUARE_FILE(pawn);

                if(pawn_file == (0x4) || pawn_file == (0xB))
                    {
                    king = KING_POS(board, opp);
                    prom = PAWN_PROMOTE(pawn, me);

                    if(DISTANCE(king, prom) <= 1 && !bishop_can_attack(board, prom, me))
                        {
                        mul[me] = 0;
                        }
                    }
                }
            }

        // K(B)P+K+ draw
        if((mat_info->cflags[me]&(1 << 1)) != 0)
            {
            pawn = pawn_info->single_file[me];

            if(pawn != 0)
                { // all pawns on one file
                king = KING_POS(board, opp);

                if(SQUARE_FILE(king) == SQUARE_FILE(pawn) && PAWN_RANK(king, me) > PAWN_RANK(pawn, me)
                    && !bishop_can_attack(board, king, me))
                    {
                    mul[me] = 0;
                    }
                }
            }

        // KNPK* draw
        if((mat_info->cflags[me]&(1 << 2)) != 0)
            {
            pawn = board->pawn[me][0];
            king = KING_POS(board, opp);

            if(SQUARE_FILE(king) == SQUARE_FILE(pawn) && PAWN_RANK(king, me) > PAWN_RANK(pawn, me)
                && PAWN_RANK(pawn, me) <= (0x9))
                {
                mul[me] = 0;
                }
            }
        }

    // recognisers, only heuristic draws here!
    if(false) { }
    else if(mat_info->recog == MAT_KPKQ)
        {
        // KPKQ (white)
        draw_init_list(list, board, 0);

        if(draw_kpkq(list, board->turn))
            {
            mul[0] = 0;
            mul[1] = 0;
            }
        }
    else if(mat_info->recog == MAT_KQKP)
        {
        // KPKQ (black)
        draw_init_list(list, board, 1);

        if(draw_kpkq(list, COLOR_OPP(board->turn)))
            {
            mul[0] = 0;
            mul[1] = 0;
            }
        }
    else if(mat_info->recog == MAT_KPKR)
        {
        // KPKR (white)
        draw_init_list(list, board, 0);

        if(draw_kpkr(list, board->turn))
            {
            mul[0] = 0;
            mul[1] = 0;
            }
        }
    else if(mat_info->recog == MAT_KRKP)
        {
        // KPKR (black)
        draw_init_list(list, board, 1);

        if(draw_kpkr(list, COLOR_OPP(board->turn)))
            {
            mul[0] = 0;
            mul[1] = 0;
            }
        }
    else if(mat_info->recog == MAT_KPKB)
        {
        // KPKB (white)
        draw_init_list(list, board, 0);

        if(draw_kpkb(list, board->turn))
            {
            mul[0] = 0;
            mul[1] = 0;
            }
        }
    else if(mat_info->recog == MAT_KBKP)
        {
        // KPKB (black)
        draw_init_list(list, board, 1);

        if(draw_kpkb(list, COLOR_OPP(board->turn)))
            {
            mul[0] = 0;
            mul[1] = 0;
            }
        }
    else if(mat_info->recog == MAT_KPKN)
        {
        // KPKN (white)
        draw_init_list(list, board, 0);

        if(draw_kpkn(list, board->turn))
            {
            mul[0] = 0;
            mul[1] = 0;
            }
        }
    else if(mat_info->recog == MAT_KNKP)
        {
        // KPKN (black)
        draw_init_list(list, board, 1);

        if(draw_kpkn(list, COLOR_OPP(board->turn)))
            {
            mul[0] = 0;
            mul[1] = 0;
            }
        }
    else if(mat_info->recog == MAT_KNPK)
        {
        // KNPK (white)
        draw_init_list(list, board, 0);

        if(draw_knpk(list, board->turn))
            {
            mul[0] = 0;
            mul[1] = 0;
            }
        }
    else if(mat_info->recog == MAT_KKNP)
        {
        // KNPK (black)
        draw_init_list(list, board, 1);

        if(draw_knpk(list, COLOR_OPP(board->turn)))
            {
            mul[0] = 0;
            mul[1] = 0;
            }
        }
    else if(mat_info->recog == MAT_KRPKR)
        {
        // KRPKR (white)
        draw_init_list(list, board, 0);

        if(draw_krpkr(list, board->turn))
            {
            mul[0] = 0;
            mul[1] = 0;
            }
        }
    else if(mat_info->recog == MAT_KRKRP)
        {
        // KRPKR (black)
        draw_init_list(list, board, 1);

        if(draw_krpkr(list, COLOR_OPP(board->turn)))
            {
            mul[0] = 0;
            mul[1] = 0;
            }
        }
    else if(mat_info->recog == MAT_KBPKB)
        {
        // KBPKB (white)
        draw_init_list(list, board, 0);

        if(draw_kbpkb(list, board->turn))
            {
            mul[0] = 0;
            mul[1] = 0;
            }
        }
    else if(mat_info->recog == MAT_KBKBP)
        {
        // KBPKB (black)
        draw_init_list(list, board, 1);

        if(draw_kbpkb(list, COLOR_OPP(board->turn)))
            {
            mul[0] = 0;
            mul[1] = 0;
            }
        }
    }

// eval_piece()
static void eval_piece(const board_t *board, const material_info_t *mat_info, const pawn_info_t *pawn_info,
    int *opening, int *endgame)
    {
    int color;
    int op[2], eg[2];
    int me, opp;
    int opp_flag;
    const sq_t *ptr;
    int from, to;
    int piece;
    int mobility;
    int kmobility[8];
    int capture;
    const int *unit;
    int rook_file, king_file;
    int king;
    int delta;
    int piece_nb, attackvalue;
    int king_rank, piece_rank, new_mobility, piece_file;

    // init
    for (color = 0; color < 2; color++)
        {
        op[color] = 0;
        eg[color] = 0;
        }

    // eval
    for (color = 0; color < 2; color++)
        {
        me = color;
        opp = COLOR_OPP(me);
        opp_flag = COLOR_FLAG(opp);
        unit = mobility_unit[me];
        piece_nb = 0;
        attackvalue = 0;

        // piece loop
        for (ptr = &board->piece[me][1]; (from = *ptr) != 0; ptr++)
            {
            piece = board->square[from];

            switch(PIECE_TYPE(piece))
                {
                case (1 << 4):
                    // mobility
                    mobility = -knight_mobility_unit;
                    kmobility[0] = 0;
                    kmobility[1] = 0;
                    kmobility[2] = 0;
                    kmobility[3] = 0;
                    kmobility[4] = 0;
                    kmobility[5] = 0;
                    kmobility[6] = 0;
                    kmobility[7] = 0;
                    if(me == 0)
                        {
                        if(WEAK_SQUARE_BLACK(from - 33))
                            kmobility[0] = unit[board->square[from - 33]];

                        if(WEAK_SQUARE_BLACK(from - 31))
                            kmobility[1] = unit[board->square[from - 31]];

                        if(WEAK_SQUARE_BLACK(from - 18))
                            kmobility[2] = unit[board->square[from - 18]];

                        if(WEAK_SQUARE_BLACK(from - 14))
                            kmobility[3] = unit[board->square[from - 14]];

                        if(WEAK_SQUARE_BLACK(from + 33))
                            kmobility[4] = unit[board->square[from + 33]];

                        if(WEAK_SQUARE_BLACK(from + 31))
                            kmobility[5] = unit[board->square[from + 31]];

                        if(WEAK_SQUARE_BLACK(from + 18))
                            kmobility[6] = unit[board->square[from + 18]];

                        if(WEAK_SQUARE_BLACK(from + 14))
                            kmobility[7] = unit[board->square[from + 14]];
                        }
                    else
                        {
                        if(WEAK_SQUARE_WHITE(from - 33))
                            kmobility[0] = unit[board->square[from - 33]];

                        if(WEAK_SQUARE_WHITE(from - 31))
                            kmobility[1] = unit[board->square[from - 31]];

                        if(WEAK_SQUARE_WHITE(from - 18))
                            kmobility[2] = unit[board->square[from - 18]];

                        if(WEAK_SQUARE_WHITE(from - 14))
                            kmobility[3] = unit[board->square[from - 14]];

                        if(WEAK_SQUARE_WHITE(from + 33))
                            kmobility[4] = unit[board->square[from + 33]];

                        if(WEAK_SQUARE_WHITE(from + 31))
                            kmobility[5] = unit[board->square[from + 31]];

                        if(WEAK_SQUARE_WHITE(from + 18))
                            kmobility[6] = unit[board->square[from + 18]];

                        if(WEAK_SQUARE_WHITE(from + 14))
                            kmobility[7] = unit[board->square[from + 14]];
                        }
                    kmobility[0] += kmobility[1];
                    kmobility[2] += kmobility[3];
                    kmobility[4] += kmobility[5];
                    kmobility[6] += kmobility[7];
                    kmobility[0] += kmobility[2];
                    kmobility[4] += kmobility[6];
                    mobility += kmobility[0];
                    mobility += kmobility[4];
                    op[me] += mobility * knight_mobility_opening;
                    eg[me] += mobility * knight_mobility_endgame;
                    // outpost
                    mobility = 0;
                    if(me == 0)
                        {
                        if(board->square[from - 17] == ((1 << 2) | (1 << 0)))
                            mobility += knight_outpost[me][square_to_64[from]];

                        if(board->square[from - 15] == ((1 << 2) | (1 << 0)))
                            mobility += knight_outpost[me][square_to_64[from]];
                        }
                    else
                        {
                        if(board->square[from + 17] == ((1 << 3) | (1 << 1)))
                            mobility += knight_outpost[me][square_to_64[from]];

                        if(board->square[from + 15] == ((1 << 3) | (1 << 1)))
                            mobility += knight_outpost[me][square_to_64[from]];
                        }
                    op[me] += mobility;
                    if(PAWN_RANK(from, me) >= (0x8))
                        {
                        piece_nb++;
                        attackvalue += 1;
                        }
                    break;

                case (1 << 5):
                    // mobility
                    mobility = -bishop_mobility_unit;
                    for (to = from - 17; capture=board->square[to], THROUGH(capture); to -= 17)
                        mobility += mobility_move;
                    mobility += unit[capture];
                    for (to = from - 15; capture=board->square[to], THROUGH(capture); to -= 15)
                        mobility += mobility_move;
                    mobility += unit[capture];
                    for (to = from + 15; capture=board->square[to], THROUGH(capture); to += 15)
                        mobility += mobility_move;
                    mobility += unit[capture];
                    for (to = from + 17; capture=board->square[to], THROUGH(capture); to += 17)
                        mobility += mobility_move;
                    mobility += unit[capture];
                    op[me] += mobility * bishop_mobility_opening;
                    eg[me] += mobility * bishop_mobility_endgame;
                    if(PAWN_RANK(from, me) >= (0x8))
                        {
                        piece_nb++;
                        attackvalue += 1;
                        }
                    break;

                case (1 << 6):
                    // mobility
                    mobility = -rook_mobility_unit;
                    for (to = from - 16; capture=board->square[to], THROUGH(capture); to -= 16)
                        mobility += mobility_move;
                    mobility += unit[capture];
                    for (to = from - 1; capture=board->square[to], THROUGH(capture); to -= 1)
                        mobility += mobility_move;
                    mobility += unit[capture];
                    for (to = from + 1; capture=board->square[to], THROUGH(capture); to += 1)
                        mobility += mobility_move;
                    mobility += unit[capture];
                    for (to = from + 16; capture=board->square[to], THROUGH(capture); to += 16)
                        mobility += mobility_move;
                    mobility += unit[capture];
                    op[me] += mobility * rook_mobility_opening;
                    eg[me] += mobility * rook_mobility_endgame;
                    // open file
                    if(use_rook_open_file)
                        {
                        op[me] -= rook_open_file_bonus_opening / 2;
                        eg[me] -= rook_open_file_bonus_endgame / 2;
                        rook_file = SQUARE_FILE(from);

                        if(board->pawn_file[me][rook_file] == 0)
                            { // no friendly pawn
                            op[me] += rook_semi_open_file_bonus_opening;
                            eg[me] += rook_semi_open_file_bonus_endgame;

                            if(board->pawn_file[opp][rook_file] == 0)
                                { // no enemy pawn
                                op[me] += rook_open_file_bonus_opening - rook_semi_open_file_bonus_opening;
                                eg[me] += rook_open_file_bonus_endgame - rook_semi_open_file_bonus_endgame;
                                }

                            if((mat_info->cflags[opp]&(1 << 3)) != 0)
                                {
                                king = KING_POS(board, opp);
                                king_file = SQUARE_FILE(king);
                                delta = abs(rook_file - king_file); // file distance

                                if(delta <= 1)
                                    {
                                    op[me] += rook_semi_king_file_bonus_opening;

                                    if(delta == 0)
                                        op[me] += rook_king_file_bonus_opening - rook_semi_king_file_bonus_opening;
                                    }
                                }
                            }
                        }
                    // 7th rank
                    if(PAWN_RANK(from, me) == (0xA))
                        {
                        if((pawn_info->flags[opp]&(1 << 0)) != 0 // opponent pawn on 7th rank
                        || PAWN_RANK(KING_POS(board, opp), me) == (0xB))
                            {
                            op[me] += rook_7th_rank_bonus_opening;
                            eg[me] += rook_7th_rank_bonus_endgame;
                            }
                        }
                    if(PAWN_RANK(from, me) >= (0x8))
                        {
                        piece_nb++;
                        attackvalue += 2;
                        }
                    break;

                case ((1 << 5) | (1 << 6)):
                    // mobility
                    mobility = -queen_mobility_unit;
                    for (to = from - 17; capture=board->square[to], THROUGH(capture); to -= 17)
                        mobility += mobility_move;
                    mobility += unit[capture];
                    for (to = from - 16; capture=board->square[to], THROUGH(capture); to -= 16)
                        mobility += mobility_move;
                    mobility += unit[capture];
                    for (to = from - 15; capture=board->square[to], THROUGH(capture); to -= 15)
                        mobility += mobility_move;
                    mobility += unit[capture];
                    for (to = from - 1; capture=board->square[to], THROUGH(capture); to -= 1)
                        mobility += mobility_move;
                    mobility += unit[capture];
                    for (to = from + 1; capture=board->square[to], THROUGH(capture); to += 1)
                        mobility += mobility_move;
                    mobility += unit[capture];
                    for (to = from + 15; capture=board->square[to], THROUGH(capture); to += 15)
                        mobility += mobility_move;
                    mobility += unit[capture];
                    for (to = from + 16; capture=board->square[to], THROUGH(capture); to += 16)
                        mobility += mobility_move;
                    mobility += unit[capture];
                    for (to = from + 17; capture=board->square[to], THROUGH(capture); to += 17)
                        mobility += mobility_move;
                    mobility += unit[capture];
                    op[me] += mobility * queen_mobility_opening;
                    eg[me] += mobility * queen_mobility_endgame;
                    king = KING_POS(board, opp);
                    king_file = SQUARE_FILE(king);
                    king_rank = SQUARE_RANK(king);
                    piece_file = SQUARE_FILE(from);
                    piece_rank = SQUARE_RANK(from);
                    new_mobility = 10 - ((abs(king_file - piece_file) + abs(king_rank - piece_rank)));
                    op[me] += new_mobility;
                    eg[me] += new_mobility;
                    // 7th rank
                    if(PAWN_RANK(from, me) == (0xA))
                        {
                        if((pawn_info->flags[opp]&(1 << 0)) != 0 // opponent pawn on 7th rank
                        || PAWN_RANK(KING_POS(board, opp), me) == (0xB))
                            {
                            op[me] += queen_7th_rank_opening;
                            eg[me] += queen_7th_rank_endgame;
                            }
                        }
                    if(PAWN_RANK(from, me) >= (0x8))
                        {
                        piece_nb++;
                        attackvalue += 4;
                        }
                    break;
                }
            }
        op[me] += s_space_weight[piece_nb] * attackvalue;
        }
    // update
    *opening += ((op[0] - op[1]) * piece_mobility_weight) / 256;
    *endgame += ((eg[0] - eg[1]) * piece_mobility_weight) / 256;
    }

// eval_king()
static void eval_king(const board_t *board, const material_info_t *mat_info, int *opening, int *endgame)
    {
    int color;
    int op[2], eg[2];
    int me, opp;
    int from;
    int penalty_1, penalty_2;
    int tmp;
    int penalty;
    int king;
    const sq_t *ptr;
    int piece;
    int attack_tot;
    int piece_nb;
    int king_file, king_rank, file;

    // init
    for (color = 0; color < 2; color++)
        {
        op[color] = 0;
        eg[color] = 0;
        }

    // white pawn shelter
    if((use_pawn_shelter && mat_info->cflags[0]&(1 << 3)) != 0)
        {
        king_is_safe[0] = false;
        penalty = 0;
        me = 0;

        if(king_safety)
            {
            if(board->square[(0x4A)] == ((1 << 7) | (1 << 0)) || board->square[(0x4B)] == ((1 << 7) | (1 << 0)))
                {
                if(board->square[(0x5A)] == ((1 << 2) | (1 << 0))
                    && (board->square[(0x5B)] == ((1 << 2) | (1 << 0))
                        || board->square[(0x6B)] == ((1 << 2) | (1 << 0))))
                    {
                    king_is_safe[0] = true;
                    }
                else if(board->square[(0x59)] == ((1 << 2) | (1 << 0))
                    && board->square[(0x6A)] == ((1 << 2) | (1 << 0))
                    && board->square[(0x5B)] == ((1 << 2) | (1 << 0)))
                    {
                    king_is_safe[0] = true;
                    }
                }
            else if(board->square[(0x45)] == ((1 << 7) | (1 << 0)) || board->square[(0x44)] == ((1 << 7) | (1 << 0)))
                {
                if(board->square[(0x55)] == ((1 << 2) | (1 << 0))
                    && (board->square[(0x54)] == ((1 << 2) | (1 << 0))
                        || board->square[(0x64)] == ((1 << 2) | (1 << 0))))
                    {
                    king_is_safe[0] = true;
                    }
                else if(board->square[(0x56)] == ((1 << 2) | (1 << 0))
                    && board->square[(0x65)] == ((1 << 2) | (1 << 0))
                    && board->square[(0x54)] == ((1 << 2) | (1 << 0)))
                    {
                    king_is_safe[0] = true;
                    }
                }
            }

        if(king_is_safe[0] == false && board->number[9] > 0)
            {
            // king
            penalty_1 = shelter_square(board, mat_info, KING_POS(board, me), me);
            // castling
            penalty_2 = penalty_1;

            if((board->flags &(1 << 0)) != 0)
                {
                tmp = shelter_square(board, mat_info, (0x4A), me);

                if(tmp < penalty_2)
                    penalty_2 = tmp;
                }

            if((board->flags &(1 << 1)) != 0)
                {
                tmp = shelter_square(board, mat_info, (0x45), me);

                if(tmp < penalty_2)
                    penalty_2 = tmp;
                }
            // penalty
            penalty = (penalty_1 + penalty_2) / 2;
            }

        if(use_pawn_storm)
            {
            file = SQUARE_FILE(KING_POS(board, 0));
            penalty += storm_file(board, file, 0);

            if(file != (0x4))
                penalty += storm_file(board, file - 1, 0);

            if(file != (0xB))
                penalty += storm_file(board, file + 1, 0);
            }
        op[me] -= penalty;
        }

    // black pawn shelter
    if((use_pawn_shelter && mat_info->cflags[1]&(1 << 3)) != 0)
        {
        king_is_safe[1] = false;
        penalty = 0;
        me = 1;

        if(king_safety)
            {
            if(board->square[(0xBA)] == ((1 << 7) | (1 << 1)) || board->square[(0xBB)] == ((1 << 7) | (1 << 1)))
                {
                if(board->square[(0xAA)] == ((1 << 3) | (1 << 1))
                    && (board->square[(0xAB)] == ((1 << 3) | (1 << 1))
                        || board->square[(0x9B)] == ((1 << 3) | (1 << 1))))
                    {
                    king_is_safe[1] = true;
                    }
                else if(board->square[(0xA9)] == ((1 << 3) | (1 << 1))
                    && board->square[(0x9A)] == ((1 << 3) | (1 << 1))
                    && board->square[(0xAB)] == ((1 << 3) | (1 << 1)))
                    {
                    king_is_safe[1] = true;
                    }
                }
            else if(board->square[(0xB5)] == ((1 << 7) | (1 << 1)) || board->square[(0xB4)] == ((1 << 7) | (1 << 1)))
                {
                if(board->square[(0xA5)] == ((1 << 3) | (1 << 1))
                    && (board->square[(0xA4)] == ((1 << 3) | (1 << 1))
                        || board->square[(0x94)] == ((1 << 3) | (1 << 1))))
                    {
                    king_is_safe[1] = true;
                    }
                else if(board->square[(0xA6)] == ((1 << 3) | (1 << 1))
                    && board->square[(0x95)] == ((1 << 3) | (1 << 1))
                    && board->square[(0xA4)] == ((1 << 3) | (1 << 1)))
                    {
                    king_is_safe[1] = true;
                    }
                }
            }

        if(king_is_safe[1] == false && board->number[8] > 0)
            {
            // king
            penalty_1 = shelter_square(board, mat_info, KING_POS(board, me), me);
            // castling
            penalty_2 = penalty_1;

            if((board->flags &(1 << 2)) != 0)
                {
                tmp = shelter_square(board, mat_info, (0xBA), me);

                if(tmp < penalty_2)
                    penalty_2 = tmp;
                }

            if((board->flags &(1 << 3)) != 0)
                {
                tmp = shelter_square(board, mat_info, (0xB5), me);

                if(tmp < penalty_2)
                    penalty_2 = tmp;
                }
            // penalty
            penalty = (penalty_1 + penalty_2) / 2;
            }

        if(use_pawn_storm)
            {
            file = SQUARE_FILE(KING_POS(board, 1));
            penalty += storm_file(board, file, 1);

            if(file != (0x4))
                penalty += storm_file(board, file - 1, 1);

            if(file != (0xB))
                penalty += storm_file(board, file + 1, 1);
            }
        op[me] -= penalty;
        }

    // king attacks
    if(use_king_attack)
        {
        for (color = 0; color < 2; color++)
            {
            if((mat_info->cflags[color]&(1 << 3)) != 0)
                {
                me = color;
                opp = COLOR_OPP(me);
                king = KING_POS(board, me);
                king_file = SQUARE_FILE(king);
                king_rank = SQUARE_RANK(king);
                // piece attacks
                attack_tot = 0;
                piece_nb = 0;

                for (ptr = &board->piece[opp][1]; (from = *ptr) != 0; ptr++)
                    {
                    piece = board->square[from];

                    if(piece_attack_king(board, piece, from, king))
                        {
                        piece_nb++;
                        attack_tot += king_attack_unit[piece];
                        }
                    }
                // scoring
                op[color] -= (attack_tot * 20 * king_attack_weight[piece_nb]) / 256;
                }
            }
        }
    // update
    *opening += op[0] - op[1];
    *endgame += eg[0] - eg[1];
    }

// eval_passer()
static void eval_passer(const board_t *board, const pawn_info_t *pawn_info, int *opening, int *endgame)
    {
    int color;
    int op[2], eg[2];
    int att, def;
    int bits;
    int file, rank;
    int sq;
    int min, max;
    int delta;
    int white_passed_nb, black_passed_nb;

    // init
    for (color = 0; color < 2; color++)
        {
        op[color] = 0;
        eg[color] = 0;
        }
    white_passed_nb = 0;
    black_passed_nb = 0;

    // passed pawns
    for (color = 0; color < 2; color++)
        {
        att = color;
        def = COLOR_OPP(att);

        for (bits = pawn_info->passed_bits[att]; bits != 0; bits &= bits - 1)
            {
            file = BIT_FIRST(bits);
            rank = BIT_LAST(board->pawn_file[att][file]);
            sq = SQUARE_MAKE(file, rank);

            if(COLOR_IS_BLACK(att))
                sq = SQUARE_RANK_MIRROR(sq);
            // opening scoring
            op[att] += quad(passed_pawn_bonus_opening_min, passed_pawn_bonus_opening_max, rank);
            // endgame scoring init
            min = passed_pawn_bonus_endgame_min;
            max = passed_pawn_bonus_endgame_max;
            delta = max - min;

            // "dangerous" bonus
            if(board->piece_size[def] <= 1 // defender has no piece
            && (unstoppable_passer(board, sq, att) || king_passer(board, sq, att)))
                {
                delta += unstoppable_passed_pawn_bonus;
                }
            else if(free_passer(board, sq, att))
                {
                delta += passed_pawn_bonus;
                }
            // king-distance bonus
            delta -= pawn_att_dist(sq, KING_POS(board, att), att) * attacker_distance;
            delta += pawn_def_dist(sq, KING_POS(board, def), att) * defender_distance;
            // endgame scoring
            eg[att] += min;

            if(delta > 0)
                eg[att] += quad(0, delta, rank);
            }
        }
    // update
    *opening += ((op[0] - op[1]) * passed_pawn_weight) / 256;
    *endgame += ((eg[0] - eg[1]) * passed_pawn_weight) / 256;
    }

// eval_pattern()
static void eval_pattern(const board_t *board, int *opening, int *endgame)
    {
    // trapped bishop (7th rank)
    if((board->square[(0xA4)] == ((1 << 5) | (1 << 0)) && board->square[(0x95)] == ((1 << 3) | (1 << 1)))
        || (board->square[(0xB5)] == ((1 << 5) | (1 << 0)) && board->square[(0xA6)] == ((1 << 3) | (1 << 1))))
        {
        *opening -= trapped_bishop;
        *endgame -= trapped_bishop;
        }

    if((board->square[(0xAB)] == ((1 << 5) | (1 << 0)) && board->square[(0x9A)] == ((1 << 3) | (1 << 1)))
        || (board->square[(0xBA)] == ((1 << 5) | (1 << 0)) && board->square[(0xA9)] == ((1 << 3) | (1 << 1))))
        {
        *opening -= trapped_bishop;
        *endgame -= trapped_bishop;
        }

    if((board->square[(0x54)] == ((1 << 5) | (1 << 1)) && board->square[(0x65)] == ((1 << 2) | (1 << 0)))
        || (board->square[(0x45)] == ((1 << 5) | (1 << 1)) && board->square[(0x56)] == ((1 << 2) | (1 << 0))))
        {
        *opening += trapped_bishop;
        *endgame += trapped_bishop;
        }

    if((board->square[(0x5B)] == ((1 << 5) | (1 << 1)) && board->square[(0x6A)] == ((1 << 2) | (1 << 0)))
        || (board->square[(0x4A)] == ((1 << 5) | (1 << 1)) && board->square[(0x59)] == ((1 << 2) | (1 << 0))))
        {
        *opening += trapped_bishop;
        *endgame += trapped_bishop;
        }

    // trapped bishop (6th rank)
    if(board->square[(0x94)] == ((1 << 5) | (1 << 0)) && board->square[(0x85)] == ((1 << 3) | (1 << 1)))
        {
        *opening -= trapped_bishop / 2;
        *endgame -= trapped_bishop / 2;
        }

    if(board->square[(0x9B)] == ((1 << 5) | (1 << 0)) && board->square[(0x8A)] == ((1 << 3) | (1 << 1)))
        {
        *opening -= trapped_bishop / 2;
        *endgame -= trapped_bishop / 2;
        }

    if(board->square[(0x64)] == ((1 << 5) | (1 << 1)) && board->square[(0x75)] == ((1 << 2) | (1 << 0)))
        {
        *opening += trapped_bishop / 2;
        *endgame += trapped_bishop / 2;
        }

    if(board->square[(0x6B)] == ((1 << 5) | (1 << 1)) && board->square[(0x7A)] == ((1 << 2) | (1 << 0)))
        {
        *opening += trapped_bishop / 2;
        *endgame += trapped_bishop / 2;
        }

    // blocked bishop
    if(board->square[(0x57)] == ((1 << 2) | (1 << 0)) && board->square[(0x67)] != 0
        && board->square[(0x46)] == ((1 << 5) | (1 << 0)))
        {
        *opening -= blocked_bishop;
        }

    if(board->square[(0x58)] == ((1 << 2) | (1 << 0)) && board->square[(0x68)] != 0
        && board->square[(0x49)] == ((1 << 5) | (1 << 0)))
        {
        *opening -= blocked_bishop;
        }

    if(board->square[(0xA7)] == ((1 << 3) | (1 << 1)) && board->square[(0x97)] != 0
        && board->square[(0xB6)] == ((1 << 5) | (1 << 1)))
        {
        *opening += blocked_bishop;
        }

    if(board->square[(0xA8)] == ((1 << 3) | (1 << 1)) && board->square[(0x98)] != 0
        && board->square[(0xB9)] == ((1 << 5) | (1 << 1)))
        {
        *opening += blocked_bishop;
        }

    // blocked rook
    if((board->square[(0x46)] == ((1 << 7) | (1 << 0)) || board->square[(0x45)] == ((1 << 7) | (1 << 0)))
        && (board->square[(0x44)] == ((1 << 6) | (1 << 0)) || board->square[(0x54)] == ((1 << 6) | (1 << 0))
            || board->square[(0x45)] == ((1 << 6) | (1 << 0))))
        {
        *opening -= blocked_rook_penalty;
        }

    if((board->square[(0x49)] == ((1 << 7) | (1 << 0)) || board->square[(0x4A)] == ((1 << 7) | (1 << 0)))
        && (board->square[(0x4B)] == ((1 << 6) | (1 << 0)) || board->square[(0x5B)] == ((1 << 6) | (1 << 0))
            || board->square[(0x4A)] == ((1 << 6) | (1 << 0))))
        {
        *opening -= blocked_rook_penalty;
        }

    if((board->square[(0xB6)] == ((1 << 7) | (1 << 1)) || board->square[(0xB5)] == ((1 << 7) | (1 << 1)))
        && (board->square[(0xB4)] == ((1 << 6) | (1 << 1)) || board->square[(0xA4)] == ((1 << 6) | (1 << 1))
            || board->square[(0xB5)] == ((1 << 6) | (1 << 1))))
        {
        *opening += blocked_rook_penalty;
        }

    if((board->square[(0xB9)] == ((1 << 7) | (1 << 1)) || board->square[(0xBA)] == ((1 << 7) | (1 << 1)))
        && (board->square[(0xBB)] == ((1 << 6) | (1 << 1)) || board->square[(0xAB)] == ((1 << 6) | (1 << 1))
            || board->square[(0xBA)] == ((1 << 6) | (1 << 1))))
        {
        *opening += blocked_rook_penalty;
        }
    }

// unstoppable_passer()
static bool unstoppable_passer(const board_t *board, int pawn, int color)
    {
    int me, opp;
    int file, rank;
    int king;
    int prom;
    const sq_t *ptr;
    int sq;
    int dist;
    me = color;
    opp = COLOR_OPP(me);
    file = SQUARE_FILE(pawn);
    rank = PAWN_RANK(pawn, me);
    king = KING_POS(board, opp);

    // clear promotion path?
    for (ptr = &board->piece[me][0]; (sq = *ptr) != 0; ptr++)
        {
        if(SQUARE_FILE(sq) == file && PAWN_RANK(sq, me) > rank)
            {
            return false; // "friendly" blocker
            }
        }

    // init
    if(rank == (0x5))
        {
        pawn += PAWN_MOVE_INC(me);
        rank++;
        }
    prom = PAWN_PROMOTE(pawn, me);
    dist = DISTANCE(pawn, prom);

    if(board->turn == opp)
        dist++;

    if(DISTANCE(king, prom) > dist)
        return true; // not in the square
    return false;
    }

// king_passer()
static bool king_passer(const board_t *board, int pawn, int color)
    {
    int me;
    int king;
    int file;
    int prom;
    me = color;
    king = KING_POS(board, me);
    file = SQUARE_FILE(pawn);
    prom = PAWN_PROMOTE(pawn, me);

    if(DISTANCE(king, prom) <= 1 && DISTANCE(king, pawn) <= 1
        && (SQUARE_FILE(king) != file || (file != (0x4) && file != (0xB))))
        {
        return true;
        }
    return false;
    }

// free_passer()
static bool free_passer(const board_t *board, int pawn, int color)
    {
    int me, opp;
    int inc;
    int sq;
    int move;
    me = color;
    opp = COLOR_OPP(me);
    inc = PAWN_MOVE_INC(me);
    sq = pawn + inc;

    if(board->square[sq] != 0)
        return false;
    move = MOVE_MAKE(pawn, sq);

    if(see_move(move, board) < 0)
        return false;
    return true;
    }

// pawn_att_dist()
static int pawn_att_dist(int pawn, int king, int color)
    {
    int me;
    int inc;
    int target;
    me = color;
    inc = PAWN_MOVE_INC(me);
    target = pawn + inc;
    return DISTANCE(king, target);
    }

// pawn_def_dist()
static int pawn_def_dist(int pawn, int king, int color)
    {
    int me;
    int inc;
    int target;
    me = color;
    inc = PAWN_MOVE_INC(me);
    target = pawn + inc;
    return DISTANCE(king, target);
    }

// draw_init_list()
static void draw_init_list(int list [], const board_t *board, int pawn_color)
    {
    int pos;
    int att, def;
    const sq_t *ptr;
    int sq;
    int pawn;
    int i;
    // init
    pos = 0;
    att = pawn_color;
    def = COLOR_OPP(att);

    // att
    for (ptr = &board->piece[att][0]; (sq = *ptr) != 0; ptr++)
        {
        list[pos++] = sq;
        }

    for (ptr = &board->pawn[att][0]; (sq = *ptr) != 0; ptr++)
        {
        list[pos++] = sq;
        }

    // def
    for (ptr = &board->piece[def][0]; (sq = *ptr) != 0; ptr++)
        {
        list[pos++] = sq;
        }

    for (ptr = &board->pawn[def][0]; (sq = *ptr) != 0; ptr++)
        {
        list[pos++] = sq;
        }
    // end marker
    list[pos] = 0;
    // file flip?
    pawn = board->pawn[att][0];

    if(SQUARE_FILE(pawn) >= (0x8))
        {
        for (i = 0; i < pos; i++)
            {
            list[i] = SQUARE_FILE_MIRROR(list[i]);
            }
        }

    // rank flip?
    if(COLOR_IS_BLACK(pawn_color))
        {
        for (i = 0; i < pos; i++)
            {
            list[i] = SQUARE_RANK_MIRROR(list[i]);
            }
        }
    }

// draw_kpkq()
static bool draw_kpkq(const int list [], int turn)
    {
    int wk, wp, bk, bq;
    int prom;
    int dist;
    // load
    wk = *list++;
    wp = *list++;
    bk = *list++;
    bq = *list++;

    // test
    if(false) { }
    else if(wp == (0xA4))
        {
        prom = (0xB4);
        dist = 4;

        if(wk == (0xA5) || wk == (0xB5))
            { // best case
            if(COLOR_IS_WHITE(turn))
                dist--;
            }
        else if(wk == (0xB4) || ((wk == (0xA6) || wk == (0xB6)) && bq != (0xB4)))
            { // white loses a tempo
            if(COLOR_IS_BLACK(turn) && SQUARE_FILE(bq) != (0x5))
                return false;
            }
        else
            {
            return false;
            }

        if(DISTANCE(bk, prom) > dist)
            return true;
        }
    else if(wp == (0xA6))
        {
        prom = (0xB6);
        dist = 4;

        if(false) { }
        else if(wk == (0xB6))
            {           // dist = 0
            dist++;     // self-blocking penalty

            if(COLOR_IS_WHITE(turn))
                dist--; // right-to-move bonus
            }
        else if(wk == (0xA5) || wk == (0xB5))
            {           // dist = 1, right side
            dist--;     // right-side bonus

            if(DELTA_INC_LINE(wp - bq) == wk - wp)
                dist++; // pinned-pawn penalty

            if(COLOR_IS_WHITE(turn))
                dist--; // right-to-move bonus
            }
        else if(wk == (0xA7) || wk == (0xB7))
            {           // dist = 1, wrong side
            if(DELTA_INC_LINE(wp - bq) == wk - wp)
                dist++; // pinned-pawn penalty

            if(COLOR_IS_WHITE(turn))
                dist--; // right-to-move bonus
            }
        else if((wk == (0xA4) || wk == (0xB4)) && bq != (0xB6))
            { // dist = 2, right side
            if(COLOR_IS_BLACK(turn) && SQUARE_FILE(bq) != (0x5))
                return false;
            dist--; // right-side bonus
            }
        else if((wk == (0xA8) || wk == (0xB8)) && bq != (0xB6))
            { // dist = 2, wrong side
            if(COLOR_IS_BLACK(turn) && SQUARE_FILE(bq) != (0x7))
                return false;
            }
        else
            {
            return false;
            }

        if(DISTANCE(bk, prom) > dist)
            return true;
        }
    return false;
    }

// draw_kpkr()
static bool draw_kpkr(const int list [], int turn)
    {
    int wk, wp, bk, br;
    int wk_file, wk_rank;
    int wp_file, wp_rank;
    int br_file, br_rank;
    int inc;
    int prom;
    int dist;
    // load
    wk = *list++;
    wp = *list++;
    bk = *list++;
    br = *list++;
    // init
    wk_file = SQUARE_FILE(wk);
    wk_rank = SQUARE_RANK(wk);
    wp_file = SQUARE_FILE(wp);
    wp_rank = SQUARE_RANK(wp);
    br_file = SQUARE_FILE(br);
    br_rank = SQUARE_RANK(br);
    inc = PAWN_MOVE_INC(0);
    prom = PAWN_PROMOTE(wp, 0);

    // conditions
    if(false) { }
    else if(DISTANCE(wk, wp) == 1) {
    // no-op
    }
    else if(DISTANCE(wk, wp) == 2 && abs(wk_rank - wp_rank) <= 1)
        {
        if(COLOR_IS_BLACK(turn) && br_file != (wk_file + wp_file) / 2)
            return false;
        }
    else
        {
        return false;
        }
    // white features
    dist = DISTANCE(wk, prom) + DISTANCE(wp, prom);

    if(wk == prom)
        dist++;

    if(wk == wp + inc)
        { // king on pawn's "front square"
        if(wp_file == (0x4))
            return false;
        dist++; // self-blocking penalty
        }

    // black features
    if(br_file != wp_file && br_rank != (0xB))
        {
        dist--; // misplaced-rook bonus
        }

    // test
    if(COLOR_IS_WHITE(turn))
        dist--; // right-to-move bonus

    if(DISTANCE(bk, prom) > dist)
        return true;
    return false;
    }

// draw_kpkb()
static bool draw_kpkb(const int list [], int turn)
    {
    int wk, wp, bk, bb;
    int inc;
    int end, to;
    int delta, inc_2;
    int sq;
    // load
    wk = *list++;
    wp = *list++;
    bk = *list++;
    bb = *list++;
    // blocked pawn?
    inc = PAWN_MOVE_INC(0);
    end = PAWN_PROMOTE(wp, 0) + inc;

    for (to = wp + inc; to != end; to += inc)
        {
        if(to == bb)
            return true; // direct blockade
        delta = to - bb;

        if(PSEUDO_ATTACK(((1 << 5) | (1 << 1)), delta))
            {
            inc_2 = DELTA_INC_ALL(delta);
            sq = bb;

            do
                {
                sq += inc_2;

                if(sq == to)
                    return true; // indirect blockade
                } while (sq != bk);
            }
        }

    return false;
    }

// draw_kpkn()
static bool draw_kpkn(const int list [], int turn)
    {
    int wk, wp, bk, bn;
    int inc;
    int end;
    int file;
    int sq;
    // load
    wk = *list++;
    wp = *list++;
    bk = *list++;
    bn = *list++;
    // blocked pawn?
    inc = PAWN_MOVE_INC(0);
    end = PAWN_PROMOTE(wp, 0) + inc;
    file = SQUARE_FILE(wp);

    if(file == (0x4) || file == (0xB))
        end -= inc;

    for (sq = wp + inc; sq != end; sq += inc)
        {
        if(sq == bn || PSEUDO_ATTACK(((1 << 4) | (1 << 1)), sq - bn))
            return true; // blockade
        }
    return false;
    }

// draw_knpk()
static bool draw_knpk(const int list [], int turn)
    {
    int wk, wn, wp, bk;
    // load
    wk = *list++;
    wn = *list++;
    wp = *list++;
    bk = *list++;

    // test
    if(wp == (0xA4) && DISTANCE(bk, (0xB4)) <= 1)
        return true;
    return false;
    }

// draw_krpkr()
static bool draw_krpkr(const int list [], int turn)
    {
    int wk, wr, wp, bk, br;
    int wp_file, wp_rank;
    int bk_file, bk_rank;
    int br_file, br_rank;
    int prom;
    // load
    wk = *list++;
    wr = *list++;
    wp = *list++;
    bk = *list++;
    br = *list++;
    // test
    wp_file = SQUARE_FILE(wp);
    wp_rank = SQUARE_RANK(wp);
    bk_file = SQUARE_FILE(bk);
    bk_rank = SQUARE_RANK(bk);
    br_file = SQUARE_FILE(br);
    br_rank = SQUARE_RANK(br);
    prom = PAWN_PROMOTE(wp, 0);

    if(false) { }
    else if(bk == prom)
        {
        // TODO: rook near (0x4) if wp_rank == (0x9)?
        if(br_file > wp_file)
            return true;
        }
    else if(bk_file == wp_file && bk_rank > wp_rank)
        {
        return true;
        }
    else if(wr == prom && wp_rank == (0xA) && (bk == (0xAA) || bk == (0xAB)) && br_file == wp_file)
        {
        if(br_rank <= (0x6))
            {
            if(DISTANCE(wk, wp) > 1)
                return true;
            }
        else
            { // br_rank >= (0x7)
            if(DISTANCE(wk, wp) > 2)
                return true;
            }
        }
    return false;
    }

// draw_kbpkb()
static bool draw_kbpkb(const int list [], int turn)
    {
    int wk, wb, wp, bk, bb;
    int inc;
    int end, to;
    int delta, inc_2;
    int sq;
    // load
    wk = *list++;
    wb = *list++;
    wp = *list++;
    bk = *list++;
    bb = *list++;

    // opposit color?
    if(SQUARE_COLOR(wb) == SQUARE_COLOR(bb))
        return false; // TODO
    // blocked pawn?
    inc = PAWN_MOVE_INC(0);
    end = PAWN_PROMOTE(wp, 0) + inc;

    for (to = wp + inc; to != end; to += inc)
        {
        if(to == bb)
            return true; // direct blockade
        delta = to - bb;

        if(PSEUDO_ATTACK(((1 << 5) | (1 << 1)), delta))
            {
            inc_2 = DELTA_INC_ALL(delta);
            sq = bb;

            do
                {
                sq += inc_2;

                if(sq == to)
                    return true; // indirect blockade
                } while (sq != bk);
            }
        }

    return false;
    }

// shelter_square()
static int shelter_square(const board_t *board, const material_info_t *mat_info, int square, int color)
    {
    int penalty;
    int file, rank;
    penalty = 0;
    file = SQUARE_FILE(square);
    rank = PAWN_RANK(square, color);
    penalty += shelter_file(board, file, rank, color) * 2;

    if(file != (0x4))
        penalty += shelter_file(board, file - 1, rank, color);

    if(file != (0xB))
        penalty += shelter_file(board, file + 1, rank, color);

    if(penalty == 0)
        penalty = 11; // weak back rank

    if(use_pawn_storm)
        {
        penalty += storm_file(board, file, color);

        if(file != (0x4))
            penalty += storm_file(board, file - 1, color);

        if(file != (0xB))
            penalty += storm_file(board, file + 1, color);
        }
    return penalty;
    }

// shelter_file()
static int shelter_file(const board_t *board, int file, int rank, int color)
    {
    int dist;
    int penalty;
    dist = BIT_FIRST(board->pawn_file[color][file] & bit_ge[rank]);
    dist = (0xB) - dist;
    penalty = 36 - dist * dist;
    return penalty;
    }

// storm_file()
static int storm_file(const board_t *board, int file, int color)
    {
    int dist;
    int penalty;
    dist = BIT_LAST(board->pawn_file[COLOR_OPP(color)][file]);
    penalty = 0;

    switch(dist)
        {
        case (0x7):
            penalty = pawn_storm_opening_penalty * 1;
            break;

        case (0x8):
            penalty = pawn_storm_opening_penalty * 2;
            break;

        case (0x9):
            penalty = pawn_storm_opening_penalty * 3;
            break;
        }
    return penalty;
    }

// bishop_can_attack()
static bool bishop_can_attack(const board_t *board, int to, int color)
    {
    const sq_t *ptr;
    int from;
    int piece;

    for (ptr = &board->piece[color][1]; (from = *ptr) != 0; ptr++)
        {
        piece = board->square[from];

        if(PIECE_IS_BISHOP(piece) && SQUARE_COLOR(from) == SQUARE_COLOR(to))
            {
            return true;
            }
        }
    return false;
    }
// end of evaluate.cpp
