// init.cpp

// includes
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include "board.h"
#include "eval.h"
#include "init.h"
#include "material.h"
#include "option.h"
#include "pawn.h"
#include "search.h"
#include "sort.h"

// lazy eval
bool use_lazy_eval = true;
int lazy_eval_margin = 200;

// mobility
int piece_mobility_weight = 256;
int knight_mobility_unit = 4;
int bishop_mobility_unit = 6;
int rook_mobility_unit = 7;
int queen_mobility_unit = 13;
int mobility_move = 1;
int mobility_attack = 1;
int mobility_defense = 0;
int knight_mobility_opening = 4;
int knight_mobility_endgame = 4;
int bishop_mobility_opening = 5;
int bishop_mobility_endgame = 5;
int rook_mobility_opening = 2;
int rook_mobility_endgame = 4;
int queen_mobility_opening = 2;
int queen_mobility_endgame = 4;
int king_mobility_opening = 0;
int king_mobility_endgame = 0;

// pawn hash table
bool use_pawn_table = true;
uint32 pawn_table_size = 16384;

// pawn
int pawn_structure_weight = 256;
int doubled_pawn_penalty_opening = 10;
int doubled_pawn_penalty_endgame = 20;
int isolated_pawn_penalty_opening = 10;
int isolated_pawn_penalty_opening_open = 20;
int isolated_pawn_penalty_endgame = 20;
int backward_pawn_penalty_opening = 8;
int backward_pawn_penalty_opening_open = 16;
int backward_pawn_penalty_endgame = 10;

//passed pawn
int passed_pawn_weight = 256;
int candidate_passed_pawn_bonus_opening_min = 5;
int candidate_passed_pawn_bonus_opening_max = 55;
int candidate_passed_pawn_bonus_endgame_min = 10;
int candidate_passed_pawn_bonus_endgame_max = 110;
int passed_pawn_bonus = 60;
int passed_pawn_bonus_opening_min = 10;
int passed_pawn_bonus_opening_max = 70;
int passed_pawn_bonus_endgame_min = 20;
int passed_pawn_bonus_endgame_max = 140;
int protected_passed_pawn_bonus_opening = 10;
int protected_passed_pawn_bonus_endgame = 20;
int unstoppable_passed_pawn_bonus = 800;

// bishop
int trapped_bishop = 100;
int blocked_bishop = 50;

// rook
bool use_rook_open_file = true;
int rook_semi_open_file_bonus_opening = 10;
int rook_semi_open_file_bonus_endgame = 10;
int rook_open_file_bonus_opening = 20;
int rook_open_file_bonus_endgame = 20;
int rook_semi_king_file_bonus_opening = 10;
int rook_king_file_bonus_opening = 20;
int rook_7th_rank_bonus_opening = 20;
int rook_7th_rank_bonus_endgame = 40;
int blocked_rook_penalty = 50;

// queen
int queen_7th_rank_opening = 10;
int queen_7th_rank_endgame = 20;

// king
int king_safety_weight = 256;
bool king_safety = false;
int king_safety_margin = 1600;
int attacker_distance = 5;
int defender_distance = 20;
bool use_king_attack = true;
int king_attack_opening = 20;
int king_attack_opening_1 = 20;
bool use_pawn_shelter = true;
bool use_pawn_storm = true;
int pawn_storm_opening_penalty = 10;

// material
int material_weight = 256;
int pawn_value_opening = 80;
int pawn_value_endgame = 90;
int knight_value_opening = 325;
int knight_value_endgame = 325;
int bishop_value_opening = 325;
int bishop_value_endgame = 325;
int rook_value_opening = 500;
int rook_value_endgame = 500;
int queen_value_opening = 975;
int queen_value_endgame = 975;
int bishop_pair_value_opening = 50;
int bishop_pair_value_endgame = 50;
int opening_exchange_penalty = 30;
int endgame_exchange_penalty = 30;
bool use_material_imbalance = true;
int material_imbalance_weight = 256;

// material hash table
bool use_material_table = true;
uint32 material_table_size = 256;

// pst
int piece_square_weight = 256;
int pawn_file_opening = 5;
int knight_center_bonus_opening = 5;
int knight_center_bonus_endgame = 5;
int knight_rank_opening = 5;
int knight_back_rank_penalty_opening = 0;
int knight_trapped_penalty = 100;
int bishop_center_bonus_opening = 2;
int bishop_center_bonus_endgame = 3;
int bishop_back_rank_penalty_opening = 10;
int bishop_diagonal_opening = 4;
int rook_file_opening = 3;
int queen_center_bonus_opening = 0;
int queen_center_bonus_endgame = 4;
int queen_back_rank_penalty_opening = 5;
int king_center_endgame = 12;
int king_file_opening = 10;
int king_rank_opening = 10;

// search
bool use_cpu_time = false;
bool use_event = true;
bool use_short_search = true;
int short_search_depth = 1;
bool display_best = true;
bool display_depth_start = true;
bool display_depth_end = true;
bool display_currmove = true;
bool display_stats = true;
bool use_easy = true;
int easy_search_threshold = 150;
double easy_search_ratio = 0.20;
bool use_early_search = true;
double early_search_ratio = 0.60;
bool use_bad = true;
int bad_threshold = 50;
bool use_extension = true;
int window_size = 20;
bool use_distance_pruning = true;

// transposition table
bool use_trans_table = true;
int trans_table_depth = 1;
bool use_exact = true;
bool use_mate_values = true;

// null move
bool use_null_move = true;
bool use_null_move_eval = false;
int null_move_depth = 2;
int null_move_reduction = 3;
bool use_verification_search = true;
bool use_verification_search_endgame = false;
int verification_search_reduction = 5;

// internal iterative deepening
bool use_iid = true;
int iid_depth = 3;
int iid_reduction = 2;

// razoring
bool use_razoring = true;
int razor_depth = 3;
int razor_margin = 300;

// history pruning
bool use_history_pruning = true;
int history_pruning_depth = 3;
int history_pruning_pv_depth = 3;
int history_pruning_move_nb = 3;
int history_pruning_threshold = 70;
bool use_extended_history_pruning = true;
bool history_pruning_research = true;

// futility pruning
bool use_futility_pruning = true;
int futility_pruning_margin = 100;
int futility_pruning_margin_1 = 100;
int futility_pruning_margin_2 = 300;
int futility_pruning_margin_3 = 950;

// quiescence search
bool use_delta_pruning = true;
int delta_pruning_margin = 50;
int quiescence_check_nb = 1;
int quiescence_check_depth = 0;

// parse_option
static void parse_option(const char *str)
    {
    char arg[4][256];
    arg[0][0] = arg[1][0] = '\0';
    sscanf(str, "%s %s", arg[0], arg[1]);

    if(!strcmp(arg[0], "//")) { }

    // lazy eval ----------------
    if(!strcmp(arg[0], "use_lazy_eval"))
        {
        int input = atoi(arg[1]);

        if(input == 1)
            {
            use_lazy_eval = true;
            printf("use_lazy_eval: %s\n", "true");
            }
        else
            {
            use_lazy_eval = false;
            printf("use_lazy_eval: %s\n", "false");
            }
        }

    if(!strcmp(arg[0], "lazy_eval_margin"))
        {
        lazy_eval_margin = atoi(arg[1]);
        printf("lazy_eval_margin: %d\n", lazy_eval_margin);
        }

    // mobility ----------------
    if(!strcmp(arg[0], "piece_mobility_weight"))
        {
        piece_mobility_weight = atoi(arg[1]) * 256 / 100;
        printf("piece_mobility_weight: %d\n", piece_mobility_weight);
        }

    if(!strcmp(arg[0], "knight_mobility_unit"))
        {
        knight_mobility_unit = atoi(arg[1]);
        printf("knight_mobility_unit: %d\n", knight_mobility_unit);
        }

    if(!strcmp(arg[0], "bishop_mobility_unit"))
        {
        bishop_mobility_unit = atoi(arg[1]);
        printf("bishop_mobility_unit: %d\n", bishop_mobility_unit);
        }

    if(!strcmp(arg[0], "rook_mobility_unit"))
        {
        rook_mobility_unit = atoi(arg[1]);
        printf("rook_mobility_unit: %d\n", rook_mobility_unit);
        }

    if(!strcmp(arg[0], "queen_mobility_unit"))
        {
        queen_mobility_unit = atoi(arg[1]);
        printf("queen_mobility_unit: %d\n", queen_mobility_unit);
        }

    if(!strcmp(arg[0], "mobility_move"))
        {
        mobility_move = atoi(arg[1]);
        printf("mobility_move: %d\n", mobility_move);
        }

    if(!strcmp(arg[0], "mobility_attack"))
        {
        mobility_attack = atoi(arg[1]);
        printf("mobility_attack: %d\n", mobility_attack);
        }

    if(!strcmp(arg[0], "mobility_defense"))
        {
        mobility_defense = atoi(arg[1]);
        printf("mobility_defense: %d\n", mobility_defense);
        }

    if(!strcmp(arg[0], "knight_mobility_opening"))
        {
        knight_mobility_opening = atoi(arg[1]);
        printf("knight_mobility_opening: %d\n", knight_mobility_opening);
        }

    if(!strcmp(arg[0], "knight_mobility_endgame"))
        {
        knight_mobility_endgame = atoi(arg[1]);
        printf("knight_mobility_endgame: %d\n", knight_mobility_endgame);
        }

    if(!strcmp(arg[0], "bishop_mobility_opening"))
        {
        bishop_mobility_opening = atoi(arg[1]);
        printf("bishop_mobility_opening: %d\n", bishop_mobility_opening);
        }

    if(!strcmp(arg[0], "bishop_mobility_endgame"))
        {
        bishop_mobility_endgame = atoi(arg[1]);
        printf("bishop_mobility_endgame: %d\n", bishop_mobility_endgame);
        }

    if(!strcmp(arg[0], "rook_mobility_opening"))
        {
        rook_mobility_opening = atoi(arg[1]);
        printf("rook_mobility_opening: %d\n", rook_mobility_opening);
        }

    if(!strcmp(arg[0], "rook_mobility_endgame"))
        {
        rook_mobility_endgame = atoi(arg[1]);
        printf("rook_mobility_endgame: %d\n", rook_mobility_endgame);
        }

    if(!strcmp(arg[0], "queen_mobility_opening"))
        {
        queen_mobility_opening = atoi(arg[1]);
        printf("queen_mobility_opening: %d\n", queen_mobility_opening);
        }

    if(!strcmp(arg[0], "queen_mobility_endgame"))
        {
        queen_mobility_endgame = atoi(arg[1]);
        printf("queen_mobility_endgame: %d\n", queen_mobility_endgame);
        }

    if(!strcmp(arg[0], "king_mobility_opening"))
        {
        king_mobility_opening = atoi(arg[1]);
        printf("king_mobility_opening: %d\n", king_mobility_opening);
        }

    if(!strcmp(arg[0], "king_mobility_endgame"))
        {
        king_mobility_endgame = atoi(arg[1]);
        printf("king_mobility_endgame: %d\n", king_mobility_endgame);
        }

    // pawn ----------------
    if(!strcmp(arg[0], "pawn_structure_weight"))
        {
        pawn_structure_weight = atoi(arg[1]) * 256 / 100;
        printf("pawn_structure_weight: %d\n", pawn_structure_weight);
        }

    if(!strcmp(arg[0], "passed_pawn_weight"))
        {
        passed_pawn_weight = atoi(arg[1]) * 256 / 100;
        printf("passed_pawn_weight: %d\n", passed_pawn_weight);
        }

    if(!strcmp(arg[0], "use_pawn_table"))
        {
        int input = atoi(arg[1]);

        if(input == 1)
            {
            use_pawn_table = true;
            printf("use_pawn_table: %s\n", "true");
            }
        else
            {
            use_pawn_table = false;
            printf("use_pawn_table: %s\n", "false");
            }
        }

    if(!strcmp(arg[0], "pawn_table_size"))
        {
        pawn_table_size = atoi(arg[1]);
        printf("pawn_table_size: %d\n", pawn_table_size);
        }

    if(!strcmp(arg[0], "doubled_pawn_penalty_opening"))
        {
        doubled_pawn_penalty_opening = atoi(arg[1]);
        printf("doubled_pawn_penalty_opening: %d\n", doubled_pawn_penalty_opening);
        }

    if(!strcmp(arg[0], "doubled_pawn_penalty_endgame"))
        {
        doubled_pawn_penalty_endgame = atoi(arg[1]);
        printf("doubled_pawn_penalty_endgame: %d\n", doubled_pawn_penalty_endgame);
        }

    if(!strcmp(arg[0], "isolated_pawn_penalty_opening"))
        {
        isolated_pawn_penalty_opening = atoi(arg[1]);
        printf("isolated_pawn_penalty_opening: %d\n", isolated_pawn_penalty_opening);
        }

    if(!strcmp(arg[0], "isolated_pawn_penalty_opening_open"))
        {
        isolated_pawn_penalty_opening_open = atoi(arg[1]);
        printf("isolated_pawn_penalty_opening_open: %d\n", isolated_pawn_penalty_opening_open);
        }

    if(!strcmp(arg[0], "isolated_pawn_penalty_endgame"))
        {
        isolated_pawn_penalty_endgame = atoi(arg[1]);
        printf("isolated_pawn_penalty_endgame: %d\n", isolated_pawn_penalty_endgame);
        }

    if(!strcmp(arg[0], "backward_pawn_penalty_opening"))
        {
        backward_pawn_penalty_opening = atoi(arg[1]);
        printf("backward_pawn_penalty_opening: %d\n", backward_pawn_penalty_opening);
        }

    if(!strcmp(arg[0], "backward_pawn_penalty_opening_open"))
        {
        backward_pawn_penalty_opening_open = atoi(arg[1]);
        printf("backward_pawn_penalty_opening_open: %d\n", backward_pawn_penalty_opening_open);
        }

    if(!strcmp(arg[0], "backward_pawn_penalty_endgame"))
        {
        backward_pawn_penalty_endgame = atoi(arg[1]);
        printf("backward_pawn_penalty_endgame: %d\n", backward_pawn_penalty_endgame);
        }

    if(!strcmp(arg[0], "candidate_passed_pawn_bonus_opening_min"))
        {
        candidate_passed_pawn_bonus_opening_min = atoi(arg[1]);
        printf("candidate_passed_pawn_bonus_opening_min: %d\n", candidate_passed_pawn_bonus_opening_min);
        }

    if(!strcmp(arg[0], "candidate_passed_pawn_bonus_opening_max"))
        {
        candidate_passed_pawn_bonus_opening_max = atoi(arg[1]);
        printf("candidate_passed_pawn_bonus_opening_max: %d\n", candidate_passed_pawn_bonus_opening_max);
        }

    if(!strcmp(arg[0], "candidate_passed_pawn_bonus_endgame_min"))
        {
        candidate_passed_pawn_bonus_endgame_min = atoi(arg[1]);
        printf("candidate_passed_pawn_bonus_endgame_min: %d\n", candidate_passed_pawn_bonus_endgame_min);
        }

    if(!strcmp(arg[0], "candidate_passed_pawn_bonus_endgame_max"))
        {
        candidate_passed_pawn_bonus_endgame_max = atoi(arg[1]);
        printf("candidate_passed_pawn_bonus_endgame_max: %d\n", candidate_passed_pawn_bonus_endgame_max);
        }

    if(!strcmp(arg[0], "protected_passed_pawn_bonus_opening"))
        {
        protected_passed_pawn_bonus_opening = atoi(arg[1]);
        printf("protected_passed_pawn_bonus_opening %d\n", protected_passed_pawn_bonus_opening);
        }

    if(!strcmp(arg[0], "protected_passed_pawn_bonus_endgame"))
        {
        protected_passed_pawn_bonus_endgame = atoi(arg[1]);
        printf("protected_passed_pawn_bonus_endgame: %d\n", protected_passed_pawn_bonus_endgame);
        }

    if(!strcmp(arg[0], "passed_pawn_bonus_opening_min"))
        {
        passed_pawn_bonus_opening_min = atoi(arg[1]);
        printf("passed_pawn_bonus_opening_min: %d\n", passed_pawn_bonus_opening_min);
        }

    if(!strcmp(arg[0], "passed_pawn_bonus_opening_max"))
        {
        passed_pawn_bonus_opening_max = atoi(arg[1]);
        printf("passed_pawn_bonus_opening_max: %d\n", passed_pawn_bonus_opening_max);
        }

    if(!strcmp(arg[0], "passed_pawn_bonus_endgame_min"))
        {
        passed_pawn_bonus_endgame_min = atoi(arg[1]);
        printf("passed_pawn_bonus_endgame_min: %d\n", passed_pawn_bonus_endgame_min);
        }

    if(!strcmp(arg[0], "passed_pawn_bonus_endgame_max"))
        {
        passed_pawn_bonus_endgame_max = atoi(arg[1]);
        printf("passed_pawn_bonus_endgame_max: %d\n", passed_pawn_bonus_endgame_max);
        }

    if(!strcmp(arg[0], "unstoppable_passed_pawn_bonus"))
        {
        unstoppable_passed_pawn_bonus = atoi(arg[1]);
        printf("unstoppable_passed_pawn_bonus: %d\n", unstoppable_passed_pawn_bonus);
        }

    if(!strcmp(arg[0], "passed_pawn_bonus"))
        {
        passed_pawn_bonus = atoi(arg[1]);
        printf("passed_pawn_bonus: %d\n", passed_pawn_bonus);
        }

    // bishop ----------------
    if(!strcmp(arg[0], "trapped_bishop"))
        {
        trapped_bishop = atoi(arg[1]);
        printf("trapped_bishop: %d\n", trapped_bishop);
        }

    if(!strcmp(arg[0], "blocked_bishop"))
        {
        blocked_bishop = atoi(arg[1]);
        printf("blocked_bishop: %d\n", blocked_bishop);
        }

    // rook ----------------
    if(!strcmp(arg[0], "use_rook_open_file"))
        {
        int input = atoi(arg[1]);

        if(input == 1)
            {
            use_rook_open_file = true;
            printf("use_rook_open_file: %s\n", "true");
            }
        else
            {
            use_rook_open_file = false;
            printf("use_rook_open_file: %s\n", "false");
            }
        }

    if(!strcmp(arg[0], "rook_semi_open_file_bonus_opening"))
        {
        rook_semi_open_file_bonus_opening = atoi(arg[1]);
        printf("rook_semi_open_file_bonus_opening: %d\n", rook_semi_open_file_bonus_opening);
        }

    if(!strcmp(arg[0], "rook_semi_open_file_bonus_endgame"))
        {
        rook_semi_open_file_bonus_endgame = atoi(arg[1]);
        printf("rook_semi_open_file_bonus_endgame: %d\n", rook_semi_open_file_bonus_endgame);
        }

    if(!strcmp(arg[0], "rook_open_file_bonus_opening"))
        {
        rook_open_file_bonus_opening = atoi(arg[1]);
        printf("rook_open_file_bonus_opening: %d\n", rook_open_file_bonus_opening);
        }

    if(!strcmp(arg[0], "rook_open_file_bonus_endgame"))
        {
        rook_open_file_bonus_endgame = atoi(arg[1]);
        printf("rook_open_file_bonus_endgame: %d\n", rook_open_file_bonus_endgame);
        }

    if(!strcmp(arg[0], "rook_semi_king_file_bonus_opening"))
        {
        rook_semi_king_file_bonus_opening = atoi(arg[1]);
        printf("rook_semi_king_file_bonus_opening: %d\n", rook_semi_king_file_bonus_opening);
        }

    if(!strcmp(arg[0], "rook_king_file_bonus_opening"))
        {
        rook_king_file_bonus_opening = atoi(arg[1]);
        printf("rook_king_file_bonus_opening: %d\n", rook_king_file_bonus_opening);
        }

    if(!strcmp(arg[0], "rook_7th_rank_bonus_opening"))
        {
        rook_7th_rank_bonus_opening = atoi(arg[1]);
        printf("rook_7th_rank_bonus_opening: %d\n", rook_7th_rank_bonus_opening);
        }

    if(!strcmp(arg[0], "rook_7th_rank_bonus_endgame"))
        {
        rook_7th_rank_bonus_endgame = atoi(arg[1]);
        printf("rook_7th_rank_bonus_endgame: %d\n", rook_7th_rank_bonus_endgame);
        }

    if(!strcmp(arg[0], "blocked_rook_penalty"))
        {
        blocked_rook_penalty = atoi(arg[1]);
        printf("blocked_rook_penalty: %d\n", blocked_rook_penalty);
        }

    // queen ----------------
    if(!strcmp(arg[0], "queen_7th_rank_opening"))
        {
        queen_7th_rank_opening = atoi(arg[1]);
        printf("queen_7th_rank_opening: %d\n", queen_7th_rank_opening);
        }

    if(!strcmp(arg[0], "queen_7th_rank_endgame"))
        {
        queen_7th_rank_endgame = atoi(arg[1]);
        printf("queen_7th_rank_endgame: %d\n", queen_7th_rank_endgame);
        }

    // king ----------------
    if(!strcmp(arg[0], "king_safety_weight"))
        {
        king_safety_weight = atoi(arg[1]) * 256 / 100;
        printf("king_safety_weight: %d\n", king_safety_weight);
        }

    if(!strcmp(arg[0], "king_safety"))
        {
        int input = atoi(arg[1]);

        if(input == 1)
            {
            king_safety = true;
            printf("king_safety: %s\n", "true");
            }
        else
            {
            king_safety = false;
            printf("king_safety: %s\n", "false");
            }
        }

    if(!strcmp(arg[0], "king_safety_margin"))
        {
        king_safety_margin = atoi(arg[1]);
        printf("king_safety_margin: %d\n", king_safety_margin);
        }

    if(!strcmp(arg[0], "attacker_distance"))
        {
        attacker_distance = atoi(arg[1]);
        printf("attacker_distance: %d\n", attacker_distance);
        }

    if(!strcmp(arg[0], "defender_distance"))
        {
        defender_distance = atoi(arg[1]);
        printf("defender_distance: %d\n", defender_distance);
        }

    if(!strcmp(arg[0], "use_king_attack"))
        {
        int input = atoi(arg[1]);

        if(input == 1)
            {
            use_king_attack = true;
            printf("use_king_attack: %s\n", "true");
            }
        else
            {
            use_king_attack = false;
            printf("use_king_attack: %s\n", "false");
            }
        }

    if(!strcmp(arg[0], "king_attack_opening"))
        {
        king_attack_opening = atoi(arg[1]);
        printf("king_attack_opening: %d\n", king_attack_opening);
        }

    if(!strcmp(arg[0], "king_attack_opening_1"))
        {
        king_attack_opening_1 = atoi(arg[1]);
        printf("king_attack_opening_1: %d\n", king_attack_opening_1);
        }

    if(!strcmp(arg[0], "use_pawn_shelter"))
        {
        int input = atoi(arg[1]);

        if(input == 1)
            {
            use_pawn_shelter = true;
            printf("use_pawn_shelter: %s\n", "true");
            }
        else
            {
            use_pawn_shelter = false;
            printf("use_pawn_shelter: %s\n", "false");
            }
        }

    if(!strcmp(arg[0], "use_pawn_storm"))
        {
        int input = atoi(arg[1]);

        if(input == 1)
            {
            use_pawn_storm = true;
            printf("use_pawn_storm: %s\n", "true");
            }
        else
            {
            use_pawn_storm = false;
            printf("use_pawn_storm: %s\n", "false");
            }
        }

    if(!strcmp(arg[0], "pawn_storm_opening_penalty"))
        {
        pawn_storm_opening_penalty = atoi(arg[1]);
        printf("pawn_storm_opening_penalty: %d\n", pawn_storm_opening_penalty);
        }

    // material ----------------
    if(!strcmp(arg[0], "material_weight"))
        {
        material_weight = atoi(arg[1]) * 256 / 100;
        printf("material_weight: %d\n", material_weight);
        }

    if(!strcmp(arg[0], "pawn_value_opening"))
        {
        pawn_value_opening = atoi(arg[1]);
        printf("pawn_value_opening: %d\n", pawn_value_opening);
        }

    if(!strcmp(arg[0], "pawn_value_endgame"))
        {
        pawn_value_endgame = atoi(arg[1]);
        printf("pawn_value_endgame: %d\n", pawn_value_endgame);
        }

    if(!strcmp(arg[0], "knight_value_opening"))
        {
        knight_value_opening = atoi(arg[1]);
        printf("knight_value_opening: %d\n", knight_value_opening);
        }

    if(!strcmp(arg[0], "knight_value_endgame"))
        {
        knight_value_endgame = atoi(arg[1]);
        printf("knight_value_endgame: %d\n", knight_value_endgame);
        }

    if(!strcmp(arg[0], "bishop_value_opening"))
        {
        bishop_value_opening = atoi(arg[1]);
        printf("bishop_value_opening: %d\n", bishop_value_opening);
        }

    if(!strcmp(arg[0], "bishop_value_endgame"))
        {
        bishop_value_endgame = atoi(arg[1]);
        printf("bishop_value_endgame: %d\n", bishop_value_endgame);
        }

    if(!strcmp(arg[0], "rook_value_opening"))
        {
        rook_value_opening = atoi(arg[1]);
        printf("rook_value_opening: %d\n", rook_value_opening);
        }

    if(!strcmp(arg[0], "rook_value_endgame"))
        {
        rook_value_endgame = atoi(arg[1]);
        printf("rook_value_endgame: %d\n", rook_value_endgame);
        }

    if(!strcmp(arg[0], "queen_value_opening"))
        {
        queen_value_opening = atoi(arg[1]);
        printf("queen_value_opening: %d\n", queen_value_opening);
        }

    if(!strcmp(arg[0], "queen_value_endgame"))
        {
        queen_value_endgame = atoi(arg[1]);
        printf("queen_value_endgame: %d\n", queen_value_endgame);
        }

    if(!strcmp(arg[0], "bishop_pair_value_opening"))
        {
        bishop_pair_value_opening = atoi(arg[1]);
        printf("bishop_pair_value_opening: %d\n", bishop_pair_value_opening);
        }

    if(!strcmp(arg[0], "bishop_pair_value_endgame"))
        {
        bishop_pair_value_endgame = atoi(arg[1]);
        printf("bishop_pair_value_endgame: %d\n", bishop_pair_value_endgame);
        }

    if(!strcmp(arg[0], "opening_exchange_penalty"))
        {
        opening_exchange_penalty = atoi(arg[1]);
        printf("opening_exchange_penalty: %d\n", opening_exchange_penalty);
        }

    if(!strcmp(arg[0], "endgame_exchange_penalty"))
        {
        endgame_exchange_penalty = atoi(arg[1]);
        printf("endgame_exchange_penalty: %d\n", endgame_exchange_penalty);
        }

    if(!strcmp(arg[0], "use_material_imbalance"))
        {
        int input = atoi(arg[1]);

        if(input == 1)
            {
            use_material_imbalance = true;
            printf("use_material_imbalance: %s\n", "true");
            }
        else
            {
            use_material_imbalance = false;
            printf("use_material_imbalance: %s\n", "false");
            }
        }

    if(!strcmp(arg[0], "material_imbalance_weight"))
        {
        material_imbalance_weight = atoi(arg[1]) * 256 / 100;
        printf("material_imbalance_weight: %d\n", material_imbalance_weight);
        }

    // material hash table ----------------
    if(!strcmp(arg[0], "use_material_table"))
        {
        int input = atoi(arg[1]);

        if(input == 1)
            {
            use_material_table = true;
            printf("use_material_table: %s\n", "true");
            }
        else
            {
            use_material_table = false;
            printf("use_material_table: %s\n", "false");
            }
        }

    if(!strcmp(arg[0], "material_table_size"))
        {
        material_table_size = atoi(arg[1]);
        printf("material_table_size: %d\n", material_table_size);
        }

    // pst ----------------
    if(!strcmp(arg[0], "piece_square_weight"))
        {
        piece_square_weight = atoi(arg[1]) * 256 / 100;
        printf("piece_square_weight: %d\n", piece_square_weight);
        }

    if(!strcmp(arg[0], "pawn_file_opening"))
        {
        pawn_file_opening = atoi(arg[1]);
        printf("pawn_file_opening: %d\n", pawn_file_opening);
        }

    if(!strcmp(arg[0], "knight_center_bonus_opening"))
        {
        knight_center_bonus_opening = atoi(arg[1]);
        printf("knight_center_bonus_opening: %d\n", knight_center_bonus_opening);
        }

    if(!strcmp(arg[0], "knight_center_bonus_endgame"))
        {
        knight_center_bonus_endgame = atoi(arg[1]);
        printf("knight_center_bonus_endgame: %d\n", knight_center_bonus_endgame);
        }

    if(!strcmp(arg[0], "knight_rank_opening"))
        {
        knight_rank_opening = atoi(arg[1]);
        printf("knight_rank_opening: %d\n", knight_rank_opening);
        }

    if(!strcmp(arg[0], "knight_back_rank_penalty_opening"))
        {
        knight_back_rank_penalty_opening = atoi(arg[1]);
        printf("knight_back_rank_penalty_opening: %d\n", knight_back_rank_penalty_opening);
        }

    if(!strcmp(arg[0], "knight_trapped_penalty"))
        {
        knight_trapped_penalty = atoi(arg[1]);
        printf("knight_trapped_penalty: %d\n", knight_trapped_penalty);
        }

    if(!strcmp(arg[0], "bishop_center_bonus_opening"))
        {
        bishop_center_bonus_opening = atoi(arg[1]);
        printf("bishop_center_bonus_opening: %d\n", bishop_center_bonus_opening);
        }

    if(!strcmp(arg[0], "bishop_back_rank_penalty_opening"))
        {
        bishop_back_rank_penalty_opening = atoi(arg[1]);
        printf("bishop_back_rank_penalty_opening: %d\n", bishop_back_rank_penalty_opening);
        }

    if(!strcmp(arg[0], "bishop_diagonal_opening"))
        {
        bishop_diagonal_opening = atoi(arg[1]);
        printf("bishop_diagonal_opening: %d\n", bishop_diagonal_opening);
        }

    if(!strcmp(arg[0], "rook_file_opening"))
        {
        rook_file_opening = atoi(arg[1]);
        printf("rook_file_opening: %d\n", rook_file_opening);
        }

    if(!strcmp(arg[0], "queen_center_bonus_opening"))
        {
        queen_center_bonus_opening = atoi(arg[1]);
        printf("queen_center_bonus_opening: %d\n", queen_center_bonus_opening);
        }

    if(!strcmp(arg[0], "queen_center_bonus_endgame"))
        {
        queen_center_bonus_endgame = atoi(arg[1]);
        printf("queen_center_bonus_endgame: %d\n", queen_center_bonus_endgame);
        }

    if(!strcmp(arg[0], "queen_back_rank_penalty_opening"))
        {
        queen_back_rank_penalty_opening = atoi(arg[1]);
        printf("queen_back_rank_penalty_opening: %d\n", queen_back_rank_penalty_opening);
        }

    if(!strcmp(arg[0], "king_center_endgame"))
        {
        king_center_endgame = atoi(arg[1]);
        printf("king_center_endgame: %d\n", king_center_endgame);
        }

    if(!strcmp(arg[0], "king_file_opening"))
        {
        king_file_opening = atoi(arg[1]);
        printf("king_file_opening: %d\n", king_file_opening);
        }

    if(!strcmp(arg[0], "king_rank_opening"))
        {
        king_rank_opening = atoi(arg[1]);
        printf("king_rank_opening: %d\n", king_rank_opening);
        }

    // search ----------------
    if(!strcmp(arg[0], "use_cpu_time"))
        {
        int input = atoi(arg[1]);

        if(input == 1)
            {
            use_cpu_time = true;
            printf("use_cpu_time: %s\n", "true");
            }
        else
            {
            use_cpu_time = false;
            printf("use_cpu_time: %s\n", "false");
            }
        }

    if(!strcmp(arg[0], "use_event"))
        {
        int input = atoi(arg[1]);

        if(input == 1)
            {
            use_event = true;
            printf("use_event: %s\n", "true");
            }
        else
            {
            use_event = false;
            printf("use_event: %s\n", "false");
            }
        }

    if(!strcmp(arg[0], "use_short_search"))
        {
        int input = atoi(arg[1]);

        if(input == 1)
            {
            use_short_search = true;
            printf("use_short_search: %s\n", "true");
            }
        else
            {
            use_short_search = false;
            printf("use_short_search: %s\n", "false");
            }
        }

    if(!strcmp(arg[0], "short_search_depth"))
        {
        short_search_depth = atoi(arg[1]);
        printf("short_search_depth: %d\n", short_search_depth);
        }

    if(!strcmp(arg[0], "display_best"))
        {
        int input = atoi(arg[1]);

        if(input == 1)
            {
            display_best = true;
            printf("display_best: %s\n", "true");
            }
        else
            {
            display_best = false;
            printf("display_best: %s\n", "false");
            }
        }

    if(!strcmp(arg[0], "display_depth_start"))
        {
        int input = atoi(arg[1]);

        if(input == 1)
            {
            display_depth_start = true;
            printf("display_depth_start: %s\n", "true");
            }
        else
            {
            display_depth_start = false;
            printf("display_depth_start: %s\n", "false");
            }
        }

    if(!strcmp(arg[0], "display_currmove"))
        {
        int input = atoi(arg[1]);

        if(input == 1)
            {
            display_currmove = true;
            printf("display_currmove: %s\n", "true");
            }
        else
            {
            display_currmove = false;
            printf("display_currmove: %s\n", "false");
            }
        }

    if(!strcmp(arg[0], "display_stats"))
        {
        int input = atoi(arg[1]);

        if(input == 1)
            {
            display_stats = true;
            printf("display_stats: %s\n", "true");
            }
        else
            {
            display_stats = false;
            printf("display_stats: %s\n", "false");
            }
        }

    if(!strcmp(arg[0], "use_easy"))
        {
        int input = atoi(arg[1]);

        if(input == 1)
            {
            use_easy = true;
            printf("use_easy: %s\n", "true");
            }
        else
            {
            use_easy = false;
            printf("use_easy: %s\n", "false");
            }
        }

    if(!strcmp(arg[0], "easy_search_threshold"))
        {
        easy_search_threshold = atoi(arg[1]);
        printf("easy_search_threshold: %d\n", easy_search_threshold);
        }

    if(!strcmp(arg[0], "easy_search_ratio"))
        {
        easy_search_ratio = double(atof(arg[1]));
        printf("easy_search_ratio: %.2f\n", easy_search_ratio);
        }

    if(!strcmp(arg[0], "use_early_search"))
        {
        int input = atoi(arg[1]);

        if(input == 1)
            {
            use_early_search = true;
            printf("use_early_search: %s\n", "true");
            }
        else
            {
            use_early_search = false;
            printf("use_early_search: %s\n", "false");
            }
        }

    if(!strcmp(arg[0], "early_search_ratio"))
        {
        early_search_ratio = double(atof(arg[1]));
        printf("early_search_ratio: %.2f\n", early_search_ratio);
        }

    if(!strcmp(arg[0], "use_bad"))
        {
        int input = atoi(arg[1]);

        if(input == 1)
            {
            use_bad = true;
            printf("use_bad: %s\n", "true");
            }
        else
            {
            use_bad = false;
            printf("use_bad: %s\n", "false");
            }
        }

    if(!strcmp(arg[0], "bad_threshold"))
        {
        bad_threshold = atoi(arg[1]);
        printf("bad_threshold: %d\n", bad_threshold);
        }

    if(!strcmp(arg[0], "use_extension"))
        {
        int input = atoi(arg[1]);

        if(input == 1)
            {
            use_extension = true;
            printf("use_extension: %s\n", "true");
            }
        else
            {
            use_extension = false;
            printf("use_extension: %s\n", "false");
            }
        }

    if(!strcmp(arg[0], "window_size"))
        {
        window_size = atoi(arg[1]);
        printf("window_size: %d\n", window_size);
        }

    if(!strcmp(arg[0], "use_distance_pruning"))
        {
        int input = atoi(arg[1]);

        if(input == 1)
            {
            use_distance_pruning = true;
            printf("use_distance_pruning: %s\n", "true");
            }
        else
            {
            use_distance_pruning = false;
            printf("use_distance_pruning: %s\n", "false");
            }
        }

    // transposition table ----------------
    if(!strcmp(arg[0], "use_trans_table"))
        {
        int input = atoi(arg[1]);

        if(input == 1)
            {
            use_trans_table = true;
            printf("use_trans_table: %s\n", "true");
            }
        else
            {
            use_trans_table = false;
            printf("use_trans_table: %s\n", "false");
            }
        }

    if(!strcmp(arg[0], "trans_table_depth"))
        {
        trans_table_depth = atoi(arg[1]);
        printf("trans_table_depth: %d\n", trans_table_depth);
        }

    if(!strcmp(arg[0], "use_exact"))
        {
        int input = atoi(arg[1]);

        if(input == 1)
            {
            use_exact = true;
            printf("use_exact: %s\n", "true");
            }
        else
            {
            use_exact = false;
            printf("use_exact: %s\n", "false");
            }
        }

    if(!strcmp(arg[0], "use_mate_values"))
        {
        int input = atoi(arg[1]);

        if(input == 1)
            {
            use_mate_values = true;
            printf("use_mate_values: %s\n", "true");
            }
        else
            {
            use_mate_values = false;
            printf("use_mate_values: %s\n", "false");
            }
        }

    // null move ----------------
    if(!strcmp(arg[0], "use_null_move"))
        {
        int input = atoi(arg[1]);

        if(input == 1)
            {
            use_null_move = true;
            printf("use_null_move: %s\n", "true");
            }
        else
            {
            use_null_move = false;
            printf("use_null_move: %s\n", "false");
            }
        }

    if(!strcmp(arg[0], "use_null_move_eval"))
        {
        int input = atoi(arg[1]);

        if(input == 1)
            {
            use_null_move_eval = true;
            printf("use_null_move_eval: %s\n", "true");
            }
        else
            {
            use_null_move_eval = false;
            printf("use_null_move_eval: %s\n", "false");
            }
        }

    if(!strcmp(arg[0], "null_move_depth"))
        {
        null_move_depth = atoi(arg[1]);
        printf("null_move_depth: %d\n", null_move_depth);
        }

    if(!strcmp(arg[0], "null_move_reduction"))
        {
        null_move_reduction = atoi(arg[1]);
        printf("null_move_reduction: %d\n", null_move_reduction);
        }

    if(!strcmp(arg[0], "use_verification_search"))
        {
        int input = atoi(arg[1]);

        if(input == 1)
            {
            use_null_move_eval = true;
            printf("use_null_move_eval: %s\n", "true");
            }
        else
            {
            use_null_move_eval = false;
            printf("use_null_move_eval: %s\n", "false");
            }
        }

    if(!strcmp(arg[0], "use_verification_search_endgame"))
        {
        int input = atoi(arg[1]);

        if(input == 1)
            {
            use_verification_search_endgame = true;
            printf("use_verification_search_endgame: %s\n", "true");
            }
        else
            {
            use_verification_search_endgame = false;
            printf("use_verification_search_endgame: %s\n", "false");
            }
        }

    if(!strcmp(arg[0], "verification_search_reduction"))
        {
        verification_search_reduction = atoi(arg[1]);
        printf("verification_search_reduction: %d\n", verification_search_reduction);
        }

    // internal iterative deepening ----------------
    if(!strcmp(arg[0], "use_iid"))
        {
        int input = atoi(arg[1]);

        if(input == 1)
            {
            use_iid = true;
            printf("use_iid: %s\n", "true");
            }
        else
            {
            use_iid = false;
            printf("use_iid: %s\n", "false");
            }
        }

    if(!strcmp(arg[0], "iid_depth"))
        {
        iid_depth = atoi(arg[1]);
        printf("iid_depth: %d\n", iid_depth);
        }

    if(!strcmp(arg[0], "iid_reduction"))
        {
        iid_reduction = atoi(arg[1]);
        printf("iid_reduction: %d\n", iid_reduction);
        }

    // razoring ----------------
    if(!strcmp(arg[0], "use_razoring"))
        {
        int input = atoi(arg[1]);

        if(input == 1)
            {
            use_razoring = true;
            printf("use_razoring: %s\n", "true");
            }
        else
            {
            use_razoring = false;
            printf("use_razoring: %s\n", "false");
            }
        }

    if(!strcmp(arg[0], "razor_depth"))
        {
        razor_depth = atoi(arg[1]);
        printf("razor_depth: %d\n", razor_depth);
        }

    if(!strcmp(arg[0], "razor_margin"))
        {
        razor_margin = atoi(arg[1]);
        printf("razor_margin: %d\n", razor_margin);
        }

    // history pruning ----------------
    if(!strcmp(arg[0], "use_history_pruning"))
        {
        int input = atoi(arg[1]);

        if(input == 1)
            {
            use_history_pruning = true;
            printf("use_history_pruning: %s\n", "true");
            }
        else
            {
            use_history_pruning = false;
            printf("use_history_pruning: %s\n", "false");
            }
        }

    if(!strcmp(arg[0], "history_pruning_depth"))
        {
        history_pruning_depth = atoi(arg[1]);
        printf("history_pruning_depth: %d\n", history_pruning_depth);
        }

    if(!strcmp(arg[0], "history_pruning_pv_depth"))
        {
        history_pruning_pv_depth = atoi(arg[1]);
        printf("history_pruning_pv_depth: %d\n", history_pruning_pv_depth);
        }

    if(!strcmp(arg[0], "history_pruning_move_nb"))
        {
        history_pruning_move_nb = atoi(arg[1]);
        printf("history_pruning_move_nb: %d\n", history_pruning_move_nb);
        }

    if(!strcmp(arg[0], "history_pruning_threshold"))
        {
        history_pruning_threshold = atoi(arg[1]);
        printf("history_pruning_threshold: %d\n", history_pruning_threshold);
        }

    if(!strcmp(arg[0], "use_extended_history_pruning"))
        {
        int input = atoi(arg[1]);

        if(input == 1)
            {
            use_extended_history_pruning = true;
            printf("use_extended_history_pruning: %s\n", "true");
            }
        else
            {
            use_extended_history_pruning = false;
            printf("use_extended_history_pruning: %s\n", "false");
            }
        }

    if(!strcmp(arg[0], "history_pruning_research"))
        {
        int input = atoi(arg[1]);

        if(input == 1)
            {
            history_pruning_research = true;
            printf("history_pruning_research: %s\n", "true");
            }
        else
            {
            history_pruning_research = false;
            printf("history_pruning_research: %s\n", "false");
            }
        }

    // futility pruning ----------------
    if(!strcmp(arg[0], "use_futility_pruning"))
        {
        int input = atoi(arg[1]);

        if(input == 1)
            {
            use_futility_pruning = true;
            printf("use_futility_pruning: %s\n", "true");
            }
        else
            {
            use_futility_pruning = false;
            printf("use_futility_pruning: %s\n", "false");
            }
        }

    if(!strcmp(arg[0], "futility_pruning_margin"))
        {
        futility_pruning_margin = atoi(arg[1]);
        printf("futility_pruning_margin: %d\n", futility_pruning_margin);
        }

    if(!strcmp(arg[0], "futility_pruning_margin_1"))
        {
        futility_pruning_margin_1 = atoi(arg[1]);
        printf("futility_pruning_margin_1: %d\n", futility_pruning_margin_1);
        }

    if(!strcmp(arg[0], "futility_pruning_margin_2"))
        {
        futility_pruning_margin_2 = atoi(arg[1]);
        printf("futility_pruning_margin_2: %d\n", futility_pruning_margin_2);
        }

    if(!strcmp(arg[0], "futility_pruning_margin_3"))
        {
        futility_pruning_margin_3 = atoi(arg[1]);
        printf("futility_pruning_margin_3: %d\n", futility_pruning_margin_3);
        }

    // quiescence search ----------------
    if(!strcmp(arg[0], "use_delta_pruning"))
        {
        int input = atoi(arg[1]);

        if(input == 1)
            {
            use_delta_pruning = true;
            printf("use_delta_pruning: %s\n", "true");
            }
        else
            {
            use_delta_pruning = false;
            printf("use_delta_pruning: %s\n", "false");
            }
        }

    if(!strcmp(arg[0], "delta_pruning_margin"))
        {
        delta_pruning_margin = atoi(arg[1]);
        printf("delta_pruning_margin: %d\n", delta_pruning_margin);
        }

    if(!strcmp(arg[0], "quiescence_check_nb"))
        {
        quiescence_check_nb = atoi(arg[1]);
        printf("quiescence_check_nb: %d\n", quiescence_check_nb);
        }

    if(!strcmp(arg[0], "quiescence_check_depth"))
        {
        quiescence_check_depth = atoi(arg[1]);
        printf("quiescence_check_depth: %d\n", quiescence_check_depth);
        }
    }

// read_ini_file()
void read_ini_file(char *file_name)
    {
    char parambuf[256];
    FILE *iniFile;
    iniFile = fopen(file_name, "rt");

    if(iniFile)
        {
        printf("%s found:\n", "togatoy.cfg");

        while(!feof(iniFile))
            {
            strcpy(parambuf, "");
            fgets(parambuf, 256, iniFile);
            parse_option(parambuf);
            }
        fclose(iniFile);
        iniFile = NULL;
        printf("\n");
        }
    else
        {
        printf("%s not found:\n", "togatoy.cfg");
        printf("using default values...\n\n");
        printf("type default to generate a default togatoy.cfg file.\n");
        printf("type random to generate a random togatoy.cfg file.\n\n");
        printf("WARNING: using a random togatoy.cfg file will very\n");
		printf("likely weaken the engine significantly. this option\n");
        printf("is provided for experimental testing purposes only.\n\n");
        }
    }

// gen_default_ini_file()
void gen_default_ini_file(char *file_name)
    {
	printf("writing default togatoy.cfg...\n");
    FILE *fp;
    fp = fopen("togatoy.cfg", "w");
    fprintf(fp, "// TogaToy 4.0 default parameters file\n");
    fprintf(fp, "\n");
    fprintf(fp, "// lazy eval\n");
    fprintf(fp, "use_lazy_eval %d\n", use_lazy_eval);
    fprintf(fp, "lazy_eval_margin %d\n", lazy_eval_margin);
    fprintf(fp, "\n");
    fprintf(fp, "// mobility\n");
    fprintf(fp, "piece_mobility_weight %d\n", piece_mobility_weight / 256 * 100);
    fprintf(fp, "knight_mobility_unit %d\n", knight_mobility_unit);
    fprintf(fp, "bishop_mobility_unit %d\n", bishop_mobility_unit);
    fprintf(fp, "rook_mobility_unit %d\n", rook_mobility_unit);
    fprintf(fp, "queen_mobility_unit %d\n", queen_mobility_unit);
    fprintf(fp, "mobility_move %d\n", mobility_move);
    fprintf(fp, "mobility_attack %d\n", mobility_attack);
    fprintf(fp, "mobility_defense %d\n", mobility_defense);
    fprintf(fp, "knight_mobility_opening %d\n", knight_mobility_opening);
    fprintf(fp, "knight_mobility_endgame %d\n", knight_mobility_endgame);
    fprintf(fp, "bishop_mobility_opening %d\n", bishop_mobility_opening);
    fprintf(fp, "bishop_mobility_endgame %d\n", bishop_mobility_endgame);
    fprintf(fp, "rook_mobility_opening %d\n", rook_mobility_opening);
    fprintf(fp, "rook_mobility_endgame %d\n", rook_mobility_endgame);
    fprintf(fp, "queen_mobility_opening %d\n", queen_mobility_opening);
    fprintf(fp, "queen_mobility_endgame %d\n", queen_mobility_endgame);
    fprintf(fp, "king_mobility_opening %d\n", king_mobility_opening);
    fprintf(fp, "king_mobility_endgame %d\n", king_mobility_endgame);
    fprintf(fp, "\n");
    fprintf(fp, "// pawn hash table\n");
	fprintf(fp, "use_pawn_table %d\n", use_pawn_table);
	fprintf(fp, "pawn_table_size %d\n", pawn_table_size);
    fprintf(fp, "\n");
    fprintf(fp, "// pawn\n");
	fprintf(fp, "pawn_structure_weight %d\n", pawn_structure_weight / 256 * 100);
	fprintf(fp, "doubled_pawn_penalty_opening %d\n", doubled_pawn_penalty_opening);
	fprintf(fp, "doubled_pawn_penalty_endgame %d\n", doubled_pawn_penalty_endgame);
	fprintf(fp, "isolated_pawn_penalty_opening %d\n", isolated_pawn_penalty_opening);
	fprintf(fp, "isolated_pawn_penalty_opening_open %d\n", isolated_pawn_penalty_opening_open);
	fprintf(fp, "isolated_pawn_penalty_endgame %d\n", isolated_pawn_penalty_endgame);
	fprintf(fp, "backward_pawn_penalty_opening %d\n", backward_pawn_penalty_opening);
	fprintf(fp, "backward_pawn_penalty_opening_open %d\n",backward_pawn_penalty_opening_open);
	fprintf(fp, "backward_pawn_penalty_endgame %d\n", backward_pawn_penalty_endgame);
    fprintf(fp, "\n");
	fprintf(fp, "// passed pawn\n");
	fprintf(fp, "passed_pawn_weight %d\n", passed_pawn_weight / 256 * 100);
	fprintf(fp, "candidate_passed_pawn_bonus_opening_min %d\n", candidate_passed_pawn_bonus_opening_min);
	fprintf(fp, "candidate_passed_pawn_bonus_opening_max %d\n", candidate_passed_pawn_bonus_opening_max);
	fprintf(fp, "candidate_passed_pawn_bonus_endgame_min %d\n", candidate_passed_pawn_bonus_endgame_min);
	fprintf(fp, "candidate_passed_pawn_bonus_endgame_max %d\n", candidate_passed_pawn_bonus_endgame_max);
	fprintf(fp, "passed_pawn_bonus %d\n", passed_pawn_bonus);
	fprintf(fp, "passed_pawn_bonus_opening_min %d\n", passed_pawn_bonus_opening_min);
	fprintf(fp, "passed_pawn_bonus_opening_max %d\n", passed_pawn_bonus_opening_max);
	fprintf(fp, "passed_pawn_bonus_endgame_min %d\n", passed_pawn_bonus_endgame_min);
	fprintf(fp, "passed_pawn_bonus_endgame_max %d\n", passed_pawn_bonus_endgame_max);
	fprintf(fp, "protected_passed_pawn_bonus_opening %d\n", protected_passed_pawn_bonus_opening);
	fprintf(fp, "protected_passed_pawn_bonus_endgame %d\n", protected_passed_pawn_bonus_endgame);
	fprintf(fp, "unstoppable_passed_pawn_bonus %d\n", unstoppable_passed_pawn_bonus);
    fprintf(fp, "\n");
    fprintf(fp, "// bishop\n");
	fprintf(fp, "trapped_bishop %d\n", trapped_bishop);
	fprintf(fp, "blocked_bishop %d\n", blocked_bishop);
    fprintf(fp, "\n");
    fprintf(fp, "// rook\n");
	fprintf(fp, "use_rook_open_file %d\n", use_rook_open_file);
	fprintf(fp, "rook_semi_open_file_bonus_opening %d\n", rook_semi_open_file_bonus_opening);
	fprintf(fp, "rook_semi_open_file_bonus_endgame %d\n", rook_semi_open_file_bonus_endgame);
	fprintf(fp, "rook_open_file_bonus_opening %d\n", rook_open_file_bonus_opening);
	fprintf(fp, "rook_open_file_bonus_endgame %d\n", rook_open_file_bonus_endgame);
	fprintf(fp, "rook_semi_king_file_bonus_opening %d\n", rook_semi_king_file_bonus_opening);
	fprintf(fp, "rook_king_file_bonus_opening %d\n", rook_king_file_bonus_opening);
	fprintf(fp, "rook_7th_rank_bonus_opening %d\n", rook_7th_rank_bonus_opening);
	fprintf(fp, "rook_7th_rank_bonus_endgame %d\n", rook_7th_rank_bonus_endgame);
	fprintf(fp, "blocked_rook_penalty %d\n", blocked_rook_penalty);
    fprintf(fp, "\n");
    fprintf(fp, "// queen\n");
	fprintf(fp, "queen_7th_rank_opening %d\n", queen_7th_rank_opening);
	fprintf(fp, "queen_7th_rank_endgame %d\n", queen_7th_rank_endgame);
    fprintf(fp, "\n");
    fprintf(fp, "// king\n");
	fprintf(fp, "king_safety_weight %d\n", king_safety_weight / 256 * 100);
	fprintf(fp, "king_safety %d\n", king_safety);
	fprintf(fp, "king_safety_margin %d\n", king_safety_margin);
	fprintf(fp, "attacker_distance %d\n", attacker_distance);
	fprintf(fp, "defender_distance %d\n", defender_distance);
	fprintf(fp, "use_king_attack %d\n", use_king_attack);
	fprintf(fp, "king_attack_opening %d\n", king_attack_opening);
	fprintf(fp, "king_attack_opening_1 %d\n", king_attack_opening_1);
	fprintf(fp, "use_pawn_shelter %d\n", use_pawn_shelter);
	fprintf(fp, "use_pawn_storm %d\n", use_pawn_storm);
	fprintf(fp, "pawn_storm_opening_penalty %d\n", pawn_storm_opening_penalty);
    fprintf(fp, "\n");
    fprintf(fp, "// material\n");
	fprintf(fp, "material_weight %d\n", material_weight / 256 * 100);
	fprintf(fp, "pawn_value_opening %d\n", pawn_value_opening);
	fprintf(fp, "pawn_value_endgame %d\n", pawn_value_endgame);
	fprintf(fp, "knight_value_opening %d\n", knight_value_opening);
	fprintf(fp, "knight_value_endgame %d\n", knight_value_endgame);
	fprintf(fp, "bishop_value_opening %d\n", bishop_value_opening);
	fprintf(fp, "bishop_value_endgame %d\n", bishop_value_endgame);
	fprintf(fp, "rook_value_opening %d\n", rook_value_opening);
	fprintf(fp, "rook_value_endgame %d\n", rook_value_endgame);
	fprintf(fp, "queen_value_opening %d\n", queen_value_opening);
	fprintf(fp, "queen_value_endgame %d\n", queen_value_endgame);
	fprintf(fp, "bishop_pair_value_opening %d\n", bishop_pair_value_opening);
	fprintf(fp, "bishop_pair_value_endgame %d\n", bishop_pair_value_endgame);
	fprintf(fp, "opening_exchange_penalty %d\n", opening_exchange_penalty);
	fprintf(fp, "endgame_exchange_penalty %d\n", endgame_exchange_penalty);
	fprintf(fp, "use_material_imbalance %d\n", use_material_imbalance);
	fprintf(fp, "material_imbalance_weight %d\n", material_imbalance_weight / 256 * 100);
    fprintf(fp, "\n");
    fprintf(fp, "// material hash table\n");
	fprintf(fp, "use_material_table %d\n", use_material_table);
	fprintf(fp, "material_table_size %d\n", material_table_size);
    fprintf(fp, "\n");
    fprintf(fp, "// pst\n");
	fprintf(fp, "piece_square_weight %d\n", piece_square_weight / 256 * 100);
	fprintf(fp, "pawn_file_opening %d\n", pawn_file_opening);
	fprintf(fp, "knight_center_bonus_opening %d\n", knight_center_bonus_opening);
	fprintf(fp, "knight_center_bonus_endgame %d\n", knight_center_bonus_endgame);
	fprintf(fp, "knight_rank_opening %d\n", knight_rank_opening);
	fprintf(fp, "knight_back_rank_penalty_opening %d\n", knight_back_rank_penalty_opening);
	fprintf(fp, "knight_trapped_penalty %d\n", knight_trapped_penalty);
	fprintf(fp, "bishop_center_bonus_opening %d\n", bishop_center_bonus_opening);
	fprintf(fp, "bishop_center_bonus_endgame %d\n", bishop_center_bonus_endgame);
	fprintf(fp, "bishop_back_rank_penalty_opening %d\n", bishop_back_rank_penalty_opening);
	fprintf(fp, "bishop_diagonal_opening %d\n", bishop_diagonal_opening);
	fprintf(fp, "rook_file_opening %d\n", rook_file_opening);
	fprintf(fp, "queen_center_bonus_opening %d\n", queen_center_bonus_opening);
	fprintf(fp, "queen_center_bonus_endgame %d\n", queen_center_bonus_endgame);
	fprintf(fp, "queen_back_rank_penalty_opening %d\n", queen_back_rank_penalty_opening);
	fprintf(fp, "king_center_endgame %d\n", king_center_endgame);
	fprintf(fp, "king_file_opening %d\n", king_file_opening);
	fprintf(fp, "king_rank_opening %d\n", king_rank_opening);
    fprintf(fp, "\n");
    fprintf(fp, "// search\n");
	fprintf(fp, "use_cpu_time %d\n", use_cpu_time);
	fprintf(fp, "use_event %d\n", use_event);
	fprintf(fp, "use_short_search %d\n", use_short_search);
	fprintf(fp, "short_search_depth %d\n", short_search_depth);
	fprintf(fp, "display_best %d\n", display_best);
	fprintf(fp, "display_depth_start %d\n", display_depth_start);
	fprintf(fp, "display_depth_end %d\n", display_depth_end);
	fprintf(fp, "display_currmove %d\n", display_currmove);
	fprintf(fp, "display_stats %d\n", display_stats);
	fprintf(fp, "use_easy %d\n", use_easy);
	fprintf(fp, "easy_search_threshold %d\n", easy_search_threshold);
	fprintf(fp, "easy_search_ratio %.2f\n", easy_search_ratio);
	fprintf(fp, "use_early_search %d\n", use_early_search);
	fprintf(fp, "early_search_ratio %.2f\n", early_search_ratio);
	fprintf(fp, "use_bad %d\n", use_bad);
	fprintf(fp, "bad_threshold %d\n", bad_threshold);
	fprintf(fp, "use_extension %d\n", use_extension);
	fprintf(fp, "window_size %d\n", window_size);
	fprintf(fp, "use_distance_pruning %d\n", use_distance_pruning);
    fprintf(fp, "\n");
    fprintf(fp, "// transposition table\n");
	fprintf(fp, "use_trans_table %d\n", use_trans_table);
	fprintf(fp, "trans_table_depth %d\n", trans_table_depth);
	fprintf(fp, "use_exact %d\n", use_exact);
	fprintf(fp, "use_mate_values %d\n", use_mate_values);
    fprintf(fp, "\n");
    fprintf(fp, "// null move\n");
	fprintf(fp, "use_null_move %d\n", use_null_move);
	fprintf(fp, "use_null_move_eval %d\n", use_null_move_eval);
	fprintf(fp, "null_move_depth %d\n", null_move_depth);
	fprintf(fp, "null_move_reduction %d\n", null_move_reduction);
	fprintf(fp, "use_verification_search %d\n", use_verification_search);
	fprintf(fp, "use_verification_search_endgame %d\n", use_verification_search_endgame);
	fprintf(fp, "verification_search_reduction %d\n", verification_search_reduction);
    fprintf(fp, "\n");
    fprintf(fp, "// internal iterative deepening\n");
	fprintf(fp, "use_iid %d\n", use_iid);
	fprintf(fp, "iid_depth %d\n", iid_depth);
	fprintf(fp, "iid_reduction %d\n", iid_reduction);
    fprintf(fp, "\n");
    fprintf(fp, "// razoring\n");
	fprintf(fp, "use_razoring %d\n", use_razoring);
	fprintf(fp, "razor_depth %d\n", razor_depth);
	fprintf(fp, "razor_margin %d\n", razor_margin);
    fprintf(fp, "\n");
    fprintf(fp, "// history pruning\n");
	fprintf(fp, "use_history_pruning %d\n", use_history_pruning);
	fprintf(fp, "history_pruning_depth %d\n", history_pruning_depth);
	fprintf(fp, "history_pruning_pv_depth %d\n", history_pruning_pv_depth);
	fprintf(fp, "history_pruning_move_nb %d\n", history_pruning_move_nb);
	fprintf(fp, "history_pruning_threshold %d\n", history_pruning_threshold);
	fprintf(fp, "use_extended_history_pruning %d\n", use_extended_history_pruning);
	fprintf(fp, "history_pruning_research %d\n", history_pruning_research);
    fprintf(fp, "\n");
    fprintf(fp, "// futility pruning\n");
	fprintf(fp, "use_futility_pruning %d\n", use_futility_pruning);
	fprintf(fp, "futility_pruning_margin %d\n", futility_pruning_margin);
	fprintf(fp, "futility_pruning_margin_1 %d\n", futility_pruning_margin_1);
	fprintf(fp, "futility_pruning_margin_2 %d\n", futility_pruning_margin_2);
	fprintf(fp, "futility_pruning_margin_3 %d\n", futility_pruning_margin_3);
    fprintf(fp, "\n");
    fprintf(fp, "// quiescence search\n");
	fprintf(fp, "use_delta_pruning %d\n", use_delta_pruning);
	fprintf(fp, "delta_pruning_margin %d\n", delta_pruning_margin);
	fprintf(fp, "quiescence_check_nb %d\n", quiescence_check_nb);
	fprintf(fp, "quiescence_check_depth %d\n", quiescence_check_depth);
    fclose(fp);
	printf("done.\n\n");
    }

	// gen_random_ini_file()
void gen_random_ini_file(char *file_name)
    {
	printf("writing random togatoy.cfg...\n");
    FILE *fp;
    fp = fopen("togatoy.cfg", "w");
    fprintf(fp, "// TogaToy 4.0 random parameters file\n");
    fprintf(fp, "\n");
    fprintf(fp, "// lazy eval\n");
    fprintf(fp, "use_lazy_eval %d\n", get_rand_num(0, 1));
    fprintf(fp, "lazy_eval_margin %d\n", get_rand_num(150, 250));
    fprintf(fp, "\n");
    fprintf(fp, "// mobility\n");
    fprintf(fp, "piece_mobility_weight %d\n", get_rand_num(90, 110));
    fprintf(fp, "knight_mobility_unit %d\n", get_rand_num(3, 5));
    fprintf(fp, "bishop_mobility_unit %d\n", get_rand_num(5, 7));
    fprintf(fp, "rook_mobility_unit %d\n", get_rand_num(6, 8));
    fprintf(fp, "queen_mobility_unit %d\n", get_rand_num(12, 14));
    fprintf(fp, "mobility_move %d\n", get_rand_num(0, 2));
    fprintf(fp, "mobility_attack %d\n", get_rand_num(0, 2));
    fprintf(fp, "mobility_defense %d\n", get_rand_num(0, 1));
    fprintf(fp, "knight_mobility_opening %d\n", get_rand_num(3, 5));
    fprintf(fp, "knight_mobility_endgame %d\n", get_rand_num(3, 5));
    fprintf(fp, "bishop_mobility_opening %d\n", get_rand_num(4, 6));
    fprintf(fp, "bishop_mobility_endgame %d\n", get_rand_num(4, 6));
    fprintf(fp, "rook_mobility_opening %d\n", get_rand_num(1, 3));
    fprintf(fp, "rook_mobility_endgame %d\n", get_rand_num(3, 5));
    fprintf(fp, "queen_mobility_opening %d\n", get_rand_num(1, 3));
    fprintf(fp, "queen_mobility_endgame %d\n", get_rand_num(3, 5));
    fprintf(fp, "king_mobility_opening %d\n", get_rand_num(0, 1));
    fprintf(fp, "king_mobility_endgame %d\n", get_rand_num(0, 1));
    fprintf(fp, "\n");
    fprintf(fp, "// pawn hash table\n");
	fprintf(fp, "use_pawn_table %d\n", get_rand_num(0, 1));
	fprintf(fp, "pawn_table_size %d\n", get_rand_num(8192, 32768));
    fprintf(fp, "\n");
    fprintf(fp, "// pawn\n");
	fprintf(fp, "pawn_structure_weight %d\n", get_rand_num(90, 110));
	fprintf(fp, "doubled_pawn_penalty_opening %d\n", get_rand_num(5, 15));
	fprintf(fp, "doubled_pawn_penalty_endgame %d\n", get_rand_num(15, 25));
	fprintf(fp, "isolated_pawn_penalty_opening %d\n", get_rand_num(5, 15));
	fprintf(fp, "isolated_pawn_penalty_opening_open %d\n", get_rand_num(15, 25));
	fprintf(fp, "isolated_pawn_penalty_endgame %d\n", get_rand_num(15, 25));
	fprintf(fp, "backward_pawn_penalty_opening %d\n", get_rand_num(4, 12));
	fprintf(fp, "backward_pawn_penalty_opening_open %d\n",get_rand_num(11, 21));
	fprintf(fp, "backward_pawn_penalty_endgame %d\n", get_rand_num(5, 15));
    fprintf(fp, "\n");
	fprintf(fp, "// passed pawn\n");
	fprintf(fp, "passed_pawn_weight %d\n", get_rand_num(100, 100));
	fprintf(fp, "candidate_passed_pawn_bonus_opening_min %d\n", get_rand_num(0, 10));
	fprintf(fp, "candidate_passed_pawn_bonus_opening_max %d\n", get_rand_num(35, 75));
	fprintf(fp, "candidate_passed_pawn_bonus_endgame_min %d\n", get_rand_num(5, 15));
	fprintf(fp, "candidate_passed_pawn_bonus_endgame_max %d\n", get_rand_num(80, 140));
	fprintf(fp, "passed_pawn_bonus %d\n", get_rand_num(60, 60));
	fprintf(fp, "passed_pawn_bonus_opening_min %d\n", get_rand_num(5, 15));
	fprintf(fp, "passed_pawn_bonus_opening_max %d\n", get_rand_num(50, 90));
	fprintf(fp, "passed_pawn_bonus_endgame_min %d\n", get_rand_num(10, 30));
	fprintf(fp, "passed_pawn_bonus_endgame_max %d\n", get_rand_num(100, 180));
	fprintf(fp, "protected_passed_pawn_bonus_opening %d\n", get_rand_num(5, 15));
	fprintf(fp, "protected_passed_pawn_bonus_endgame %d\n", get_rand_num(15, 25));
	fprintf(fp, "unstoppable_passed_pawn_bonus %d\n", get_rand_num(600, 1000));
    fprintf(fp, "\n");
    fprintf(fp, "// bishop\n");
	fprintf(fp, "trapped_bishop %d\n", get_rand_num(90, 110));
	fprintf(fp, "blocked_bishop %d\n", get_rand_num(40, 60));
    fprintf(fp, "\n");
    fprintf(fp, "// rook\n");
	fprintf(fp, "use_rook_open_file %d\n", get_rand_num(0, 1));
	fprintf(fp, "rook_semi_open_file_bonus_opening %d\n", get_rand_num(5, 15));
	fprintf(fp, "rook_semi_open_file_bonus_endgame %d\n", get_rand_num(5, 15));
	fprintf(fp, "rook_open_file_bonus_opening %d\n", get_rand_num(15, 25));
	fprintf(fp, "rook_open_file_bonus_endgame %d\n", get_rand_num(15, 25));
	fprintf(fp, "rook_semi_king_file_bonus_opening %d\n", get_rand_num(5, 15));
	fprintf(fp, "rook_king_file_bonus_opening %d\n", get_rand_num(15, 25));
	fprintf(fp, "rook_7th_rank_bonus_opening %d\n", get_rand_num(15, 25));
	fprintf(fp, "rook_7th_rank_bonus_endgame %d\n", get_rand_num(30, 50));
	fprintf(fp, "blocked_rook_penalty %d\n", get_rand_num(40, 60));
    fprintf(fp, "\n");
    fprintf(fp, "// queen\n");
	fprintf(fp, "queen_7th_rank_opening %d\n", get_rand_num(5, 15));
	fprintf(fp, "queen_7th_rank_endgame %d\n", get_rand_num(15, 25));
    fprintf(fp, "\n");
    fprintf(fp, "// king\n");
	fprintf(fp, "king_safety_weight %d\n", get_rand_num(90, 110));
	fprintf(fp, "king_safety %d\n", get_rand_num(0, 1));
	fprintf(fp, "king_safety_margin %d\n", get_rand_num(1400, 1800));
	fprintf(fp, "attacker_distance %d\n", get_rand_num(3, 7));
	fprintf(fp, "defender_distance %d\n", get_rand_num(15, 25));
	fprintf(fp, "use_king_attack %d\n", get_rand_num(0, 1));
	fprintf(fp, "king_attack_opening %d\n", get_rand_num(15, 25));
	fprintf(fp, "king_attack_opening_1 %d\n", get_rand_num(15, 25));
	fprintf(fp, "use_pawn_shelter %d\n", get_rand_num(0, 1));
	fprintf(fp, "use_pawn_storm %d\n", get_rand_num(0, 1));
	fprintf(fp, "pawn_storm_opening_penalty %d\n", get_rand_num(5, 15));
    fprintf(fp, "\n");
    fprintf(fp, "// material\n");
	fprintf(fp, "material_weight %d\n", get_rand_num(90, 110));
	fprintf(fp, "pawn_value_opening %d\n", get_rand_num(70, 90));
	fprintf(fp, "pawn_value_endgame %d\n", get_rand_num(80, 100));
	fprintf(fp, "knight_value_opening %d\n", get_rand_num(300, 350));
	fprintf(fp, "knight_value_endgame %d\n", get_rand_num(300, 350));
	fprintf(fp, "bishop_value_opening %d\n", get_rand_num(300, 350));
	fprintf(fp, "bishop_value_endgame %d\n", get_rand_num(300, 350));
	fprintf(fp, "rook_value_opening %d\n", get_rand_num(480, 520));
	fprintf(fp, "rook_value_endgame %d\n", get_rand_num(480, 520));
	fprintf(fp, "queen_value_opening %d\n", get_rand_num(900, 1050));
	fprintf(fp, "queen_value_endgame %d\n", get_rand_num(900, 1050));
	fprintf(fp, "bishop_pair_value_opening %d\n", get_rand_num(30, 70));
	fprintf(fp, "bishop_pair_value_endgame %d\n", get_rand_num(40, 80));
	fprintf(fp, "opening_exchange_penalty %d\n", get_rand_num(20, 40));
	fprintf(fp, "endgame_exchange_penalty %d\n", get_rand_num(20, 40));
	fprintf(fp, "use_material_imbalance %d\n", get_rand_num(0, 1));
	fprintf(fp, "material_imbalance_weight %d\n", get_rand_num(90, 110));
    fprintf(fp, "\n");
    fprintf(fp, "// material hash table\n");
	fprintf(fp, "use_material_table %d\n", get_rand_num(0, 1));
	fprintf(fp, "material_table_size %d\n", get_rand_num(128, 512));
    fprintf(fp, "\n");
    fprintf(fp, "// pst\n");
	fprintf(fp, "piece_square_weight %d\n", get_rand_num(90, 110));
	fprintf(fp, "pawn_file_opening %d\n", get_rand_num(3, 7));
	fprintf(fp, "knight_center_bonus_opening %d\n", get_rand_num(3, 7));
	fprintf(fp, "knight_center_bonus_endgame %d\n", get_rand_num(3, 7));
	fprintf(fp, "knight_rank_opening %d\n", get_rand_num(3, 7));
	fprintf(fp, "knight_back_rank_penalty_opening %d\n", get_rand_num(0, 1));
	fprintf(fp, "knight_trapped_penalty %d\n", get_rand_num(80, 120));
	fprintf(fp, "bishop_center_bonus_opening %d\n", get_rand_num(1, 3));
	fprintf(fp, "bishop_center_bonus_endgame %d\n", get_rand_num(2, 4));
	fprintf(fp, "bishop_back_rank_penalty_opening %d\n", get_rand_num(8, 12));
	fprintf(fp, "bishop_diagonal_opening %d\n", get_rand_num(3, 5));
	fprintf(fp, "rook_file_opening %d\n", get_rand_num(2, 4));
	fprintf(fp, "queen_center_bonus_opening %d\n", get_rand_num(0, 1));
	fprintf(fp, "queen_center_bonus_endgame %d\n", get_rand_num(3, 5));
	fprintf(fp, "queen_back_rank_penalty_opening %d\n", get_rand_num(4, 6));
	fprintf(fp, "king_center_endgame %d\n", get_rand_num(10, 14));
	fprintf(fp, "king_file_opening %d\n", get_rand_num(8, 12));
	fprintf(fp, "king_rank_opening %d\n", get_rand_num(8, 12));
    fprintf(fp, "\n");
    fprintf(fp, "// search\n");
	fprintf(fp, "use_cpu_time %d\n", get_rand_num(0, 1));
	fprintf(fp, "use_event %d\n", get_rand_num(0, 1));
	fprintf(fp, "use_short_search %d\n", get_rand_num(0, 1));
	fprintf(fp, "short_search_depth %d\n", get_rand_num(0, 2));
	fprintf(fp, "display_best %d\n", get_rand_num(1, 1));
	fprintf(fp, "display_depth_start %d\n", get_rand_num(1, 1));
	fprintf(fp, "display_depth_end %d\n", get_rand_num(1, 1));
	fprintf(fp, "display_currmove %d\n", get_rand_num(1, 1));
	fprintf(fp, "display_stats %d\n", get_rand_num(1, 1));
	fprintf(fp, "use_easy %d\n", get_rand_num(0, 1));
	fprintf(fp, "easy_search_threshold %d\n", get_rand_num(120, 180));
	fprintf(fp, "easy_search_ratio %.2f\n", 0.20);
	fprintf(fp, "use_early_search %d\n", get_rand_num(0, 1));
	fprintf(fp, "early_search_ratio %.2f\n", 0.60);
	fprintf(fp, "use_bad %d\n", get_rand_num(0, 1));
	fprintf(fp, "bad_threshold %d\n", get_rand_num(30, 70));
	fprintf(fp, "use_extension %d\n", get_rand_num(1, 1));
	fprintf(fp, "window_size %d\n", get_rand_num(1, 40));
	fprintf(fp, "use_distance_pruning %d\n", get_rand_num(0, 1));
    fprintf(fp, "\n");
    fprintf(fp, "// transposition table\n");
	fprintf(fp, "use_trans_table %d\n", get_rand_num(0, 1));
	fprintf(fp, "trans_table_depth %d\n", get_rand_num(0, 2));
	fprintf(fp, "use_exact %d\n", get_rand_num(0, 1));
	fprintf(fp, "use_mate_values %d\n", get_rand_num(0, 1));
    fprintf(fp, "\n");
    fprintf(fp, "// null move\n");
	fprintf(fp, "use_null_move %d\n", get_rand_num(1, 1));
	fprintf(fp, "use_null_move_eval %d\n", get_rand_num(0, 1));
	fprintf(fp, "null_move_depth %d\n", get_rand_num(1, 3));
	fprintf(fp, "null_move_reduction %d\n", get_rand_num(2, 4));
	fprintf(fp, "use_verification_search %d\n", get_rand_num(0, 1));
	fprintf(fp, "use_verification_search_endgame %d\n", get_rand_num(0, 1));
	fprintf(fp, "verification_search_reduction %d\n", get_rand_num(4, 6));
    fprintf(fp, "\n");
    fprintf(fp, "// internal iterative deepening\n");
	fprintf(fp, "use_iid %d\n", get_rand_num(0, 1));
	fprintf(fp, "iid_depth %d\n", get_rand_num(2, 4));
	fprintf(fp, "iid_reduction %d\n", get_rand_num(1, 3));
    fprintf(fp, "\n");
    fprintf(fp, "// razoring\n");
	fprintf(fp, "use_razoring %d\n", get_rand_num(0, 1));
	fprintf(fp, "razor_depth %d\n", get_rand_num(2, 4));
	fprintf(fp, "razor_margin %d\n", get_rand_num(200, 400));
    fprintf(fp, "\n");
    fprintf(fp, "// history pruning\n");
	fprintf(fp, "use_history_pruning %d\n", get_rand_num(0, 1));
	fprintf(fp, "history_pruning_depth %d\n", get_rand_num(2, 4));
	fprintf(fp, "history_pruning_pv_depth %d\n", get_rand_num(2, 4));
	fprintf(fp, "history_pruning_move_nb %d\n", get_rand_num(2, 4));
	fprintf(fp, "history_pruning_threshold %d\n", get_rand_num(60, 90));
	fprintf(fp, "use_extended_history_pruning %d\n", get_rand_num(0, 1));
	fprintf(fp, "history_pruning_research %d\n", get_rand_num(0, 1));
    fprintf(fp, "\n");
    fprintf(fp, "// futility pruning\n");
	fprintf(fp, "use_futility_pruning %d\n", get_rand_num(0, 1));
	fprintf(fp, "futility_pruning_margin %d\n", get_rand_num(80, 120));
	fprintf(fp, "futility_pruning_margin_1 %d\n", get_rand_num(80, 120));
	fprintf(fp, "futility_pruning_margin_2 %d\n", get_rand_num(250, 350));
	fprintf(fp, "futility_pruning_margin_3 %d\n", get_rand_num(800, 1100));
    fprintf(fp, "\n");
    fprintf(fp, "// quiescence search\n");
	fprintf(fp, "use_delta_pruning %d\n", get_rand_num(0, 1));
	fprintf(fp, "delta_pruning_margin %d\n", get_rand_num(40, 60));
	fprintf(fp, "quiescence_check_nb %d\n", get_rand_num(0, 1));
	fprintf(fp, "quiescence_check_depth %d\n", get_rand_num(0, 1));
    fclose(fp);
	printf("done.\n\n");
    }

	// get_rand_num
static int get_rand_num(int min, int max)
	{
	int range;
	int r;
	unsigned first = time(NULL);
	srand(first);
	range = max - min + 1;
	r = (rand() % (range)) + min;
	return r;
	}

// end of init.cpp
