// init.h

#ifndef INIT_H
#define INIT_H

#include "util.h"

// lazy eval
extern bool use_lazy_eval;
extern int lazy_eval_margin;

//mobility
extern int piece_mobility_weight;
extern int knight_mobility_unit;
extern int bishop_mobility_unit;
extern int rook_mobility_unit;
extern int queen_mobility_unit;
extern int mobility_move;
extern int mobility_attack;
extern int mobility_defense;
extern int knight_mobility_opening;
extern int knight_mobility_endgame;
extern int bishop_mobility_opening;
extern int bishop_mobility_endgame;
extern int rook_mobility_opening;
extern int rook_mobility_endgame;
extern int queen_mobility_opening;
extern int queen_mobility_endgame;
extern int king_mobility_opening;
extern int king_mobility_endgame;

// pawn hash table
extern bool use_pawn_table;
extern uint32 pawn_table_size;

//pawn
extern int pawn_structure_weight;
extern int doubled_pawn_penalty_opening;
extern int doubled_pawn_penalty_endgame;
extern int isolated_pawn_penalty_opening;
extern int isolated_pawn_penalty_opening_open;
extern int isolated_pawn_penalty_endgame;
extern int backward_pawn_penalty_opening;
extern int backward_pawn_penalty_opening_open;
extern int backward_pawn_penalty_endgame;

//passed pawn
extern int passed_pawn_weight;
extern int candidate_passed_pawn_bonus_opening_min;
extern int candidate_passed_pawn_bonus_opening_max;
extern int candidate_passed_pawn_bonus_endgame_min;
extern int candidate_passed_pawn_bonus_endgame_max;
extern int passed_pawn_bonus;
extern int passed_pawn_bonus_opening_min;
extern int passed_pawn_bonus_opening_max;
extern int passed_pawn_bonus_endgame_min;
extern int passed_pawn_bonus_endgame_max;
extern int protected_passed_pawn_bonus_opening;
extern int protected_passed_pawn_bonus_endgame;
extern int unstoppable_passed_pawn_bonus;

//bishop
extern int trapped_bishop;
extern int blocked_bishop;

//rook
extern bool use_rook_open_file;
extern int rook_semi_open_file_bonus_opening;
extern int rook_semi_open_file_bonus_endgame;
extern int rook_open_file_bonus_opening;
extern int rook_open_file_bonus_endgame;
extern int rook_semi_king_file_bonus_opening;
extern int rook_king_file_bonus_opening;
extern int rook_7th_rank_bonus_opening;
extern int rook_7th_rank_bonus_endgame;
extern int blocked_rook_penalty;

//queen
extern int queen_7th_rank_opening;
extern int queen_7th_rank_endgame;

//king
extern int king_safety_weight;
extern bool king_safety;
extern int king_safety_margin;
extern int attacker_distance;
extern int defender_distance;
extern bool use_king_attack;
extern int king_attack_opening;
extern int king_attack_opening_1;
extern bool use_pawn_shelter;
extern bool use_pawn_storm;
extern int pawn_storm_opening_penalty;

// material
extern int material_weight;
extern int pawn_value_opening;
extern int pawn_value_endgame;
extern int knight_value_opening;
extern int knight_value_endgame;
extern int bishop_value_opening;
extern int bishop_value_endgame;
extern int rook_value_opening;
extern int rook_value_endgame;
extern int queen_value_opening;
extern int queen_value_endgame;
extern int bishop_pair_value_opening;
extern int bishop_pair_value_endgame;
extern int opening_exchange_penalty;
extern int endgame_exchange_penalty;
extern bool use_material_imbalance;
extern int material_imbalance_weight;

// material hash table
extern bool use_material_table;
extern uint32 material_table_size;

//pst
extern int piece_square_weight;
extern int pawn_file_opening;
extern int knight_center_bonus_opening;
extern int knight_center_bonus_endgame;
extern int knight_rank_opening;
extern int knight_back_rank_penalty_opening;
extern int knight_trapped_penalty;
extern int bishop_center_bonus_opening;
extern int bishop_center_bonus_endgame;
extern int bishop_back_rank_penalty_opening;
extern int bishop_diagonal_opening;
extern int rook_file_opening;
extern int queen_center_bonus_opening;
extern int queen_center_bonus_endgame;
extern int queen_back_rank_penalty_opening;
extern int king_center_endgame;
extern int king_file_opening;
extern int king_rank_opening;

// search
extern bool use_cpu_time;
extern bool use_event;
extern bool use_short_search;
extern int short_search_depth;
extern bool display_best;
extern bool display_depth_start;
extern bool display_depth_end;
extern bool display_currmove;
extern bool display_stats;
extern bool use_easy;
extern int easy_search_threshold;
extern double easy_search_ratio;
extern bool use_early_search;
extern double early_search_ratio;
extern bool use_bad;
extern int bad_threshold;
extern bool use_extension;
extern int window_size;
extern bool use_distance_pruning;

// transposition table
extern bool use_trans_table;
extern int trans_table_depth;
extern bool use_exact;
extern bool use_mate_values;

// null move
extern bool use_null_move;
extern bool use_null_move_eval;
extern int null_move_depth;
extern int null_move_reduction;
extern bool use_verification_search;
extern bool use_verification_search_endgame;
extern int verification_search_reduction;

// internal iterative deepening
extern bool use_iid;
extern int iid_depth;
extern int iid_reduction;

// razoring
extern bool use_razoring;
extern int razor_depth;
extern int razor_margin;

// history pruning
extern bool use_history_pruning;
extern int history_pruning_depth;
extern int history_pruning_pv_depth;
extern int history_pruning_move_nb;
extern int history_pruning_threshold;
extern bool use_extended_history_pruning;
extern bool history_pruning_research;

// futility pruning
extern bool use_futility_pruning;
extern int futility_pruning_margin;
extern int futility_pruning_margin_1;
extern int futility_pruning_margin_2;
extern int futility_pruning_margin_3;

// quiescence search
extern bool use_delta_pruning;
extern int delta_pruning_margin;
extern int quiescence_check_nb;
extern int quiescence_check_depth;

extern void read_ini_file(char *file_name);
extern void gen_default_ini_file(char *file_name);
extern void gen_random_ini_file(char *file_name);
static int get_rand_num(int min, int max);

#endif // !defined INIT_H
// end of init.h
