// position.cpp

// includes
#include "init.h"
#include "option.h"
#include "piece.h"
#include "pst.h"

// macros
#define P(piece_12,square_64,stage) (pst[(piece_12)][(square_64)][(stage)])

// constants
static const int pawn_file[8] =
    {
    -3, -1, +0, +1, +1, +0, -1, -3,
    };
static const int knight_line[8] =
    {
    -4, -2, +0, +1, +1, +0, -2, -4,
    };
static const int knight_rank[8] =
    {
    -2, -1, +0, +1, +2, +3, +2, +1,
    };
static const int bishop_line[8] =
    {
    -3, -1, +0, +1, +1, +0, -1, -3,
    };
static const int rook_file[8] =
    {
    -2, -1, +0, +1, +1, +0, -1, -2,
    };
static const int queen_line[8] =
    {
    -3, -1, +0, +1, +1, +0, -1, -3,
    };
static const int king_line[8] =
    {
    -3, -1, +0, +1, +1, +0, -1, -3,
    };
static const int king_file[8] =
    {
    +3, +4, +2, +0, +0, +2, +4, +3,
    };
static const int king_rank[8] =
    {
    +1, +0, -2, -3, -4, -5, -6, -7,
    };

// variables
sint16 pst[12][64][2];

// prototypes
static int square_make(int file, int rank);
static int square_file(int square);
static int square_rank(int square);
static int square_opp(int square);

// functions

// pst_init()
void pst_init()
    {
    int i;
    int piece, sq, stage;

    // init
    for (piece = 0; piece < 12; piece++)
        {
        for (sq = 0; sq < 64; sq++)
            {
            for (stage = 0; stage < 2; stage++)
                {
                P(piece, sq, stage) = 0;
                }
            }
        }
    // pawns
    piece = 0;

    // file
    for (sq = 0; sq < 64; sq++)
        {
        P(piece, sq, 0) += pawn_file[square_file(sq)] * pawn_file_opening;
        }

    for (sq = 0; sq < 64; sq++)
        {
        P(piece, sq, 0) = (P(piece, sq, 0) * pawn_structure_weight) / 256;
        P(piece, sq, 1) = (P(piece, sq, 1) * pawn_structure_weight) / 256;
        }
    // knights
    piece = 2;

    // center
    for (sq = 0; sq < 64; sq++)
        {
        P(piece, sq, 0) += knight_line[square_file(sq)] * knight_center_bonus_opening;
        P(piece, sq, 0) += knight_line[square_rank(sq)] * knight_center_bonus_opening;
        P(piece, sq, 1) += knight_line[square_file(sq)] * knight_center_bonus_endgame;
        P(piece, sq, 1) += knight_line[square_rank(sq)] * knight_center_bonus_endgame;
        }

    // rank
    for (sq = 0; sq < 64; sq++)
        {
        P(piece, sq, 0) += knight_rank[square_rank(sq)] * knight_rank_opening;
        }

    // back rank
    for (sq = (000); sq <= (007); sq++)
        {
        P(piece, sq, 0) -= knight_back_rank_penalty_opening;
        }
    // "trapped"
    P(piece, (070), 0) -= knight_trapped_penalty;
    P(piece, (077), 0) -= knight_trapped_penalty;

    for (sq = 0; sq < 64; sq++)
        {
        P(piece, sq, 0) = (P(piece, sq, 0) * piece_square_weight) / 256;
        P(piece, sq, 1) = (P(piece, sq, 1) * piece_square_weight) / 256;
        }
    // bishops
    piece = 4;

    // center
    for (sq = 0; sq < 64; sq++)
        {
        P(piece, sq, 0) += bishop_line[square_file(sq)] * bishop_center_bonus_opening;
        P(piece, sq, 0) += bishop_line[square_rank(sq)] * bishop_center_bonus_opening;
        P(piece, sq, 1) += bishop_line[square_file(sq)] * bishop_center_bonus_endgame;
        P(piece, sq, 1) += bishop_line[square_rank(sq)] * bishop_center_bonus_endgame;
        }

    // back rank
    for (sq = (000); sq <= (007); sq++)
        {
        P(piece, sq, 0) -= bishop_back_rank_penalty_opening;
        }

    // main diagonals
    for (i = 0; i < 8; i++)
        {
        sq = square_make(i, i);
        P(piece, sq, 0) += bishop_diagonal_opening;
        P(piece, square_opp(sq), 0) += bishop_diagonal_opening;
        }

    // weight
    for (sq = 0; sq < 64; sq++)
        {
        P(piece, sq, 0) = (P(piece, sq, 0) * piece_square_weight) / 256;
        P(piece, sq, 1) = (P(piece, sq, 1) * piece_square_weight) / 256;
        }
    // rooks
    piece = 6;

    // file
    for (sq = 0; sq < 64; sq++)
        {
        P(piece, sq, 0) += rook_file[square_file(sq)] * rook_file_opening;
        }

    // weight
    for (sq = 0; sq < 64; sq++)
        {
        P(piece, sq, 0) = (P(piece, sq, 0) * piece_square_weight) / 256;
        P(piece, sq, 1) = (P(piece, sq, 1) * piece_square_weight) / 256;
        }
    // queens
    piece = 8;

    // center
    for (sq = 0; sq < 64; sq++)
        {
        P(piece, sq, 0) += queen_line[square_file(sq)] * queen_center_bonus_opening;
        P(piece, sq, 0) += queen_line[square_rank(sq)] * queen_center_bonus_opening;
        P(piece, sq, 1) += queen_line[square_file(sq)] * queen_center_bonus_endgame;
        P(piece, sq, 1) += queen_line[square_rank(sq)] * queen_center_bonus_endgame;
        }

    // back rank
    for (sq = (000); sq <= (007); sq++)
        {
        P(piece, sq, 0) -= queen_back_rank_penalty_opening;
        }

    // weight
    for (sq = 0; sq < 64; sq++)
        {
        P(piece, sq, 0) = (P(piece, sq, 0) * piece_square_weight) / 256;
        P(piece, sq, 1) = (P(piece, sq, 1) * piece_square_weight) / 256;
        }
    // kings
    piece = 10;

    // center
    for (sq = 0; sq < 64; sq++)
        {
        P(piece, sq, 1) += king_line[square_file(sq)] * king_center_endgame;
        P(piece, sq, 1) += king_line[square_rank(sq)] * king_center_endgame;
        }

    // file
    for (sq = 0; sq < 64; sq++)
        {
        P(piece, sq, 0) += king_file[square_file(sq)] * king_file_opening;
        }

    // rank
    for (sq = 0; sq < 64; sq++)
        {
        P(piece, sq, 0) += king_rank[square_rank(sq)] * king_rank_opening;
        }

    // weight
    for (sq = 0; sq < 64; sq++)
        {
        P(piece, sq, 0) = (P(piece, sq, 0) * king_safety_weight) / 256;
        P(piece, sq, 1) = (P(piece, sq, 1) * piece_square_weight) / 256;
        }

    // symmetry copy for black
    for (piece = 0; piece < 12; piece += 2)
        {
        for (sq = 0; sq < 64; sq++)
            {
            for (stage = 0; stage < 2; stage++)
                {
                P(piece + 1, sq, stage) = -P(piece, square_opp(sq), stage);
                }
            }
        }
    }

// square_make()
static int square_make(int file, int rank)
    {
    return (rank << 3) | file;
    }

// square_file()
static int square_file(int square)
    {
    return square & 7;
    }

// square_rank()
static int square_rank(int square)
    {
    return square >> 3;
    }

// square_opp()
static int square_opp(int square)
    {
    return square ^ 070;
    }
// end of position.cpp
