// search.h

#ifndef SEARCH_H
#define SEARCH_H

// includes
#include <csetjmp>
#include "move_list.h"
#include "move.h"

// types
struct search_multipv_t
    {
    int mate;
    int depth;
    int max_depth;
    int value;
    double time;
    sint64 node_nb;
    char pv_string[512];
    };

struct search_param_t
    {
    int move;
    int best_move;
    int threat_move;
    bool reduced;
    };

struct search_input_t
    {
    board_t board[1];
    list_t list[1];
    bool infinite;
    bool depth_is_limited;
    int depth_limit;
    int multipv;
    volatile bool exit_engine;
    bool time_is_limited;
    double time_limit_1;
    double time_limit_2;
    bool search_moves;
    list_t moves_to_search[1];
    };

struct search_info_t
    {
    jmp_buf buf;
    bool can_stop;
    volatile bool stop;
    volatile bool stopped;
    int quiescence_check_nb;
    int check_inc;
    double last_time;
    };

struct search_root_t
    {
    list_t list[1];
    int depth;
    int move;
    int move_pos;
    int move_nb;
    int last_value;
    bool bad_1;
    bool bad_2;
    bool change;
    bool easy;
    bool flag;
    };

struct search_best_t
    {
    int move;
    int value;
    int flags;
    int depth;
    mv_t pv[256];
    };

struct search_current_t
    {
    board_t board[1];
    my_timer_t timer[1];
    int max_depth;
    int act_iteration;
    int multipv;
    sint64 node_nb;
    double time;
    double speed;
    };

// variables
extern int number_threads;
extern search_input_t search_input[1];
extern search_info_t search_info[16][1];
extern search_best_t search_best[16][10];
extern search_root_t search_root[16][1];
extern search_current_t search_current[16][1];

// functions
extern void search_clear();
extern void search();
extern void search_smp(int thread_id);
extern void search_update_best(int thread_id);
extern void search_update_root(int thread_id);
extern void search_update_current(int thread_id);
extern void search_check(int thread_id);
extern void start_suspend_threads();
extern void resume_threads();
extern void threads_init();
extern void search_full_init(list_t *list, board_t *board, int thread_id);
extern int search_full_root(list_t *list, board_t *board, int depth, int search_type, int thread_id);

#endif // !defined SEARCH_H
// end of search.h
